<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 02/09/2016
 * Time: 11:14
 */

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\FpaItemlotelanc;
use Folha\Entities\FpaLotelancevento;
use Folha\Entities\GpeMarcacaoponto;
use HCM\Util\DateUtil;
use HCM\Util\ManipulacaoExcel;
use HCM\Util\FPA;
use HCM\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Geral\Services\FileService;
use HCM\Geral\Services\UploadFileService;
use HCM\Geral\Services\ItemLoteLancService;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\Geral\Entity\FpaeventomEntity;
use HCM\FPA\Calculos\Geral\CalendarioLogic;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Repository\LoteLancEventoRepository;
use HCM\FPA\Repository\GozoFeriasRepository;
use HCM\FPA\Repository\BloqueioEventoVinculoRepository;

class LoteLancEventoService {

    const ORIGEM_BTL = 'BTL';
    const ORIGEM_FPA = 'FPA';
    const ORIGEM_PTO = 'PTO';
    const ORIGEM_SST = 'SST';
    const ORIGEM_VAB = 'VAB';
    const ORIGEM_EXCEL = 'EXCEL';
    
    const REGISTROS_EMPREG = 1;
    const REGISTROS_ESCALA = 2;
    const REGISTROS_FUNC = 3;
    const ERRO  = 1111;
    const AVISO = 1110;
    
    const REGEX_CARACTERES_ALFANUMERICOS = '/[^a-zA-Z0-9\s]/';
    const REGEX_QUEBRA_LINHA_CR_LF = "/[\n\r]/";
    const CARACTER_VAZIO = '';
    const QUEBRA_LINHA_CR_LF = "\r\n";

    /* Extensões */
    const TXT = 'TXT';

    private $nrorg;
    private $dtmescompetenc;
    private $itensLote = array();
    private $mensagens = array();
    protected $nrestruturam;
    
    protected $hcmParameters;
    protected $idoriglotelanc = 'undefined';

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, LoteLancEventoRepository $LoteLancEventoRepository, BloqueioEventoVinculoRepository $bloqueioVinculoRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->LoteLancEventoRepository = $LoteLancEventoRepository;
        $this->bloqueioEventVincRepository = $bloqueioVinculoRepository;
        $this->gpeVinculomEntity = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->calendarioLogic = new CalendarioLogic($entityManager, $hcmParameters);
        $this->_estrutura = new EstruturamEntity($entityManager, $hcmParameters);
        $this->itemLoteLanc = Repositories::FPA_ITEMLOTELANC;
        $this->gpeMarcacao = Repositories::GPE_MARCACAOPONTO;
        $this->httpKernel = $httpKernel;
    }
    
    public function deleteLote($row) {
        
        try {
            $this->entityManager->beginTransaction();
            
            if($row['IDSTATUSLOTELANC'] == 'APROVADO'){
                throw new \Exception ("Favor cancelar o lote antes de excluí-lo");
            }
            
            $lote = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_LOTELANCEVENTO)->findOneBy(array('nrorg' => $row['NRORG'],
            'nrlotelancevento' => $row['NRLOTELANCEVENTO']));
            
            $itensLote = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITEMLOTELANC)->findBy(array('nrorg' => $row['NRORG'],
            'nrlotelancevento' => $row['NRLOTELANCEVENTO']));
            
            foreach($itensLote as $iteLote){
                $this->entityManager->remove($iteLote);
            }
            
            $this->entityManager->remove($lote);
            $this->entityManager->flush();
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function deteleItensLoteZerados($row) {
        
        try {
            $this->entityManager->beginTransaction();
            
            $itensLote = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITEMLOTELANC)->findBy(array('nrorg' => $row['NRORG'],
                'nrlotelancevento' => $row['NRLOTELANCEVENTO'], 'vrreffolhalote' => 0 , 'vreventolote' => 0));  
        
            foreach($itensLote as $iteLote){
                $this->entityManager->remove($iteLote);
            }
            
            $this->entityManager->flush();
            $this->entityManager->commit();
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function setOrigemLoteLanc($origem) {
        try {
            switch ($origem) {
                case self::ORIGEM_BTL:
                    $this->idoriglotelanc = self::ORIGEM_BTL;
                    break;
                case self::ORIGEM_FPA:
                    $this->idoriglotelanc = self::ORIGEM_FPA;
                    break;
                case self::ORIGEM_PTO:
                    $this->idoriglotelanc = self::ORIGEM_PTO;
                    break;
                case self::ORIGEM_SST:
                    $this->idoriglotelanc = self::ORIGEM_SST;
                    break;
                case self::ORIGEM_VAB:
                    $this->idoriglotelanc = self::ORIGEM_VAB;
                    break;
                case self::ORIGEM_EXCEL:
                    $this->idoriglotelanc = self::ORIGEM_EXCEL;
                    break;
            }

            if ($this->idoriglotelanc === 'undefined') {
                throw new \Exception(Messages::HCM01000_FPA_LOTELANCEVEN_ORIG_UNDEFINED, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $dtmescompetencia
     * @param $origem
     * @param $dslotelanc
     * @param $idstatuslote
     * @param $dsobslote
     * @param $nrtipomoviment
     * @param $itens
     * @return $nrlote
     * @throws \Exception
     */
    public function geraLoteLancEvento($nrorg, $dtmescompetencia, $origem, $dslotelanc, $idstatuslote = 'PENDENTE', $dsobslote = null, $nrtipomoviment, $itens) {
        try {
            $this->entityManager->beginTransaction();
            $this->setOrigemLoteLanc($origem);
            $nrlote = $this->insereItensLoteLancEvento($nrorg, $itens);
            if (!is_null($nrlote)) {
                $this->insereLoteLancEvento($nrorg, $dtmescompetencia, $nrlote, $dslotelanc, $idstatuslote, $dsobslote, $nrtipomoviment);
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
            return $nrlote;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    /**
     * @param int $nrorg
     * @param array $itens
     * @param int|null $nrlote Se for passado como nulo é gerado um novo número de lote
     * @return int|null $nrlote
     * @throws \Exception
     */
    public function insereItensLoteLancEvento($nrorg, $itens, $nrlote=null)
    {
        try {
            if (!empty($itens)) {
                foreach ($itens as $info) {
                    if($info['NREVENTOM'] == 480){
                        $info['VRREFFOLHALOTE'] = null;
                    }
                    $dsobsItemlote = isset($info['DSOBSERVACAO']) ? $info['DSOBSERVACAO'] : null;
                    
                    $obj = new $this->itemLoteLanc;
                    $obj->setNrvinculom($info['NRVINCULOM']);
                    $obj->setNreventom($info['NREVENTOM']);
                    $obj->setVrreffolhalote($info['VRREFFOLHALOTE']);
                    $obj->setVreventolote($info['VREVENTOLOTE']);
                    $obj->setIdativo('S');
                    $obj->setNrorg($nrorg);
                    $obj->setNrorginclusao($nrorg);
                    $obj->setDtinclusao(DateUtil::getDataAtual());
                    $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $obj->setNrorgultatu($nrorg);
                    $obj->setDtultatu(DateUtil::getDataAtual());
                    $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $obj->setDsobservacao($dsobsItemlote);
                    if (($obj->getVreventolote()) || ($obj->getVrreffolhalote())) {
                        if (is_null($nrlote)) {
                            $nrlote = NovoCodigo::geraCodigo($nrorg, 'FPA_LOTELANCEVENTO', 1, 22, true, false);
                        }
                        $obj->setNrlotelancevento($nrlote);
                        $obj->setNritlotelanc(Novocodigo::geraCodigo($nrorg, 'FPA_ITEMLOTELANC', 1, 22, true, false));
                        $this->entityManager->persist($obj);
                    }
                }
            }
            return $nrlote;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $dtmescompetencia
     * @param $nrlote
     * @param $dslotelanc
     * @param $idstatuslote
     * @param $dsobslote
     * @param $idoriglote
     * @param $nrtipomoviment
     * @throws \Exception
     */
    public function insereLoteLancEvento($nrorg, $dtmescompetencia, $nrlote, $dslotelanc, $idstatuslote, $dsobslote, $nrtipomoviment, $nmestrutlegal = null, $nmestrutgeren = null) {
        try {
            if ($this->idoriglotelanc === 'undefined') {
                throw new \Exception(Messages::HCM01000_FPA_LOTELANCEVEN_ORIG_UNDEFINED, Messages::ERROR_MESSAGE_CODE);
            }
            if (!$nrlote) {
                $nrlote = NovoCodigo::geraCodigo($nrorg, 'FPA_LOTELANCEVENTO', 1, 22, true, false);
            }
            if(empty($dsobslote)){
                if(!empty($nmestrutlegal)){
                    $dsobslote = 'Estrutura Legal: '.$nmestrutlegal.'
';
                }
                if(!empty($nmestrutgeren)){
                    $dsobslote .= 'Estrutura Gerencial: ' .$nmestrutgeren;
                }
                
            }
            $obj = new FpaLotelancevento();
            $obj->setNrlotelancevento($nrlote);
            $obj->setDtlotelancevento(DateUtil::getDataAtual());
            $obj->setDtcompetloteeve(DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true));
            $obj->setDslotelancevento($dslotelanc);
            $obj->setIdstatuslotelanc($idstatuslote);
            $obj->setIdoriglotelanc($this->idoriglotelanc);
            $obj->setNrtipomoviment($nrtipomoviment);
            $obj->setDsobslotelanc($dsobslote);
            $obj->setIdativo('S');
            $obj->setNrorginclusao($nrorg);
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($nrorg);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorg($nrorg);
            $this->entityManager->persist($obj);
            $this->entityManager->flush($obj);
            return $obj;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function importaArquivoAcj($row){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $arquivo = $row['DSPATH'][0];
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        if (!$file->isFile()) {
            throw new \Exception("Falha ao gravar o arquivo.");
        }
        
        $arquivo = file_get_contents($file->getPathname());
        $arquivo = str_replace("\r", '', $arquivo);
        $arquivo = explode("\n", $arquivo);
        
        $registros = array();
        
        //Separa registros por codigo e funcionario.
        foreach($arquivo as $linha){
            if($linha[9] == self::REGISTROS_ESCALA){
                $registros[$linha[9]][substr($linha, 10, 4)] = $linha;
            }else if($linha[9] == self::REGISTROS_FUNC){
                $registros[$linha[9]][substr($linha, 11, 11)][] = $linha;
            }else{
                $registros[$linha[9]] = $linha;
            }
        }

        if($registros){
            $dslotelanc = 'IMPORTAÇÃO ACJEF/MTE';
            $this->idoriglotelanc = $dslotelanc;
            
            $loteLanc = $this->insereLoteLancEvento($nrorg, $mesCompetencia, null, $dslotelanc, 'PENDENTE', $dslotelanc, 1);
            $nrLote = $loteLanc->getNrlotelancevento();
            
            $dadosContrato = array();
            foreach ($registros[self::REGISTROS_ESCALA] as $hrContra) {
                $dadosContrato[substr($hrContra, 10,4)]['INI_INTERVAL'] = substr($hrContra, 18,4);
                $dadosContrato[substr($hrContra, 10,4)]['FIM_INTERVAL'] = substr($hrContra, 22,4);
            }
            
            foreach ($registros[self::REGISTROS_FUNC] as $pis => $linhas) {
                
                $vinculo = $this->gpeVinculomEntity->retornaVinculosAtivosMesCompetencia($nrorg, $mesCompetencia, null, null, null, false, null, true, 
                    false,  null,  null, null, null, $pis    
                )[0];
                
                if($vinculo){
                    $dadosVinculo = array();
                    $nrVinculom = $vinculo->getNrvinculom();
                    $nrModalidadeSalario = $this->hcmParameters->getNrModalidadeSalario();
                    $alteSalario = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->retornaAlteSalarioAtualPorVinculoModalidade(
                        $nrorg, $mesCompetencia, $nrVinculom, $nrModalidadeSalario
                    )[0];
                    
                    $nrEstrutura = $vinculo->historico->getNrestrutlegal();
                    
                    $estrutura = $this->_estrutura->retornaEstrutura($nrEstrutura);
                    
                    $nrCalendario = $estrutura->historico->getNrcalendario();
                    
                    $this->calendarioLogic->carregaFeriados($nrCalendario, DateUtil::getDataDeString($mesCompetencia), DateUtil::getUltimoDiaDoMes($mesCompetencia));
                    
                    foreach ($linhas as $linha) {
                        
                        //<------------------- Dados Dias do Mes ------------------------>
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORARIO_ENTRADA'] = substr($linha, 30,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['CDHORARIO'] = substr($linha, 34,4);
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_DIURNAS_NAO_EX'] = substr($linha, 38,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_NOTURNAS_NAO_EX'] = substr($linha, 42,4);
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_EXTRA1'] = substr($linha, 46,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['PERCENT_HE1'] = substr($linha, 50,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['MODALIDADE_HE1'] = $linha[54];
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_EXTRA2'] = substr($linha, 55,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['PERCENT_HE2'] = substr($linha, 59,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['MODALIDADE_HE2'] = $linha[63];
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_EXTRA3'] = substr($linha, 64,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['PERCENT_HE3'] = substr($linha, 68,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['MODALIDADE_HE3'] = $linha[72];
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_EXTRA4'] = substr($linha, 73,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['PERCENT_HE4'] = substr($linha, 77,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['MODALIDADE_HE4'] = $linha[81];
                        
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_FALTAS'] = substr($linha, 82,4);
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['HORAS_A_COMPENSAR'] = $linha[86];
                        $dadosVinculo['DIAS'][substr($linha, 22,8)]['SALDO_HORAS'] = substr($linha, 87,4);
                        //<------------------- Somatorios Mês ------------------------>
                        
                        $dataAtual = substr($linha, 22, 8);
                        $dia_semana = DateUtil::getDiaSemana($dataAtual);
                        
                        // Domingo E Feriado.
                        if ($dia_semana == 7 || array_key_exists($dataAtual, $this->_vetFeriados)) {
                        
                            $dadosVinculo['HORAEXTRA_DOMINGO_FERI_1'] += substr($linha, 46, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_1'] += substr($linha, 48, 2);
                            
                            $dadosVinculo['HORAEXTRA_DOMINGO_FERI_2'] += substr($linha, 55, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_2'] += substr($linha, 57, 2);
                            
                            $dadosVinculo['HORAEXTRA_DOMINGO_FERI_3'] += substr($linha, 64, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_3'] += substr($linha, 66, 2);
                            
                            $dadosVinculo['HORAEXTRA_DOMINGO_FERI_4'] += substr($linha, 73, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_4'] += substr($linha, 75, 2);
                            
                        }
                        //Sabado.
                        else if($dia_semana == 6){
                            $dadosVinculo['HORAEXTRA_SABADO_1'] += substr($linha, 46, 2);
                            $dadosVinculo['MINUTOS_EXTRA_SABADO_1'] += substr($linha, 48, 2);
                            
                            $dadosVinculo['HORAEXTRA_SABADO_2'] += substr($linha, 55, 2);
                            $dadosVinculo['MINUTOS_EXTRA_SABADO_2'] += substr($linha, 57, 2);
                            
                            $dadosVinculo['HORAEXTRA_SABADO_3'] += substr($linha, 64, 2);
                            $dadosVinculo['MINUTOS_EXTRA_SABADO_3'] += substr($linha, 66, 2);
                            
                            $dadosVinculo['HORAEXTRA_SABADO_4'] += substr($linha, 73, 2);
                            $dadosVinculo['MINUTOS_EXTRA_SABADO_4'] += substr($linha, 75, 2);
                        }
                        //DIA DE SEMANA
                        else{
                            $dadosVinculo['HORAEXTRA_DIA_SEMANA_1'] += substr($linha, 46, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_1'] += substr($linha, 48, 2);
                            
                            $dadosVinculo['HORAEXTRA_DIA_SEMANA_2'] += substr($linha, 55, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_2'] += substr($linha, 57, 2);
                            
                            $dadosVinculo['HORAEXTRA_DIA_SEMANA_3'] += substr($linha, 64, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_3'] += substr($linha, 66, 2);
                            
                            $dadosVinculo['HORAEXTRA_DIA_SEMANA_4'] += substr($linha, 73, 2);
                            $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_4'] += substr($linha, 75, 2);
                        }
                        
                        $dadosVinculo['HORAS_DIURNAS_NAO_EX_SOMA'] += substr($linha, 38, 2);
                        $dadosVinculo['MINUTOS_DIURNOS_NAO_EX_SOMA'] += substr($linha, 40, 2);
                        
                        $dadosVinculo['HORAS_NOTURNAS_NAO_EX_SOMA'] += substr($linha, 42, 2);
                        $dadosVinculo['MINUTOS_NOTURNAS_NAO_EX_SOMA'] += substr($linha, 44, 2);
                        
                        $dadosVinculo['HORAEXTRA1'] += substr($linha, 46, 2);
                        $dadosVinculo['MINUTOS_EXTRA1'] += substr($linha, 48, 2);
                        
                        $dadosVinculo['HORAEXTRA2'] += substr($linha, 55, 2);
                        $dadosVinculo['MINUTOS_EXTRA2'] += substr($linha, 57, 2);
                        
                        $dadosVinculo['HORAEXTRA3'] += substr($linha, 64, 2);
                        $dadosVinculo['MINUTOS_EXTRA3'] += substr($linha, 66, 2);
                        
                        $dadosVinculo['HORAEXTRA4'] += substr($linha, 73, 2);
                        $dadosVinculo['MINUTOS_EXTRA4'] += substr($linha, 75, 2);
                        
                        $dadosVinculo['HORAS_FALTAS_TOTAL'] += substr($linha, 82, 2);
                        $dadosVinculo['MINUTOS_FALTA_TOTAL'] += substr($linha, 84, 2);
                        
                        // <--------------------------------------------------------------->
                        
                    }
                        
                     foreach ($dadosVinculo["DIAS"] as $data => $dados) {
                        if($dados['HORARIO_ENTRADA'] != '    ') {
                            $dtEntrada = substr($data, 0, 2)."/".substr($data, 2, 2)."/".substr($data, 4);
                            $hrEntrada = substr($dados['HORARIO_ENTRADA'], 0, 2).":".substr($dados['HORARIO_ENTRADA'], 2, 2).":00";
                            $dthrEntrada = DateUtil::getDataDeString($dtEntrada." ".$hrEntrada, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                            if($dados['HORAS_A_COMPENSAR'] == '1')
                                $qthrscompensar = $this->horaToDecimal($dados['HORAS_FALTAS']);
                                
                            else 
                                $qthrscompensar = -$this->horaToDecimal($dados['HORAS_FALTAS']);
                            $this->LoteLancEventoRepository->deleteMarcacao($nrorg,$nrVinculom,$dtEntrada); //evita registros duplicados
                            $this->LoteLancEventoRepository->insertMarcacao($nrorg, $nrVinculom, $dthrEntrada, $dthrEntrada, $this->horaToDecimal($dados['HORAS_DIURNAS_NAO_EX']), $this->horaToDecimal($dados['HORAS_NOTURNAS_NAO_EX']), $this->horaToDecimal($dados['HORAS_EXTRA1']), $this->percentFloat($dados['PERCENT_HE1']), $dados['MODALIDADE_HE1'], $this->horaToDecimal($dados['HORAS_EXTRA2']), $this->percentFloat($dados['PERCENT_HE2']), $dados['MODALIDADE_HE2'], $this->horaToDecimal($dados['HORAS_EXTRA3']), $this->percentFloat($dados['PERCENT_HE3']), $dados['MODALIDADE_HE3'], $this->horaToDecimal($dados['HORAS_EXTRA4']), $this->percentFloat($dados['PERCENT_HE4']), $dados['MODALIDADE_HE4'], $this->horaToDecimal($dados['HORAS_FALTAS']), $qthrscompensar);
                        }
                    }

                    $horaExtraDomingoFeri1 =  (($dadosVinculo['HORAEXTRA_DOMINGO_FERI_1'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_1']) / 60; 
                    $horaExtraDomingoFeri2 =  (($dadosVinculo['HORAEXTRA_DOMINGO_FERI_2'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_2']) / 60; 
                    $horaExtraDomingoFeri3 =  (($dadosVinculo['HORAEXTRA_DOMINGO_FERI_3'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_3']) / 60; 
                    $horaExtraDomingoFeri4 =  (($dadosVinculo['HORAEXTRA_DOMINGO_FERI_4'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DOMINGO_FERI_4']) / 60; 
                    
                    $horaExtraSabado1 =  (($dadosVinculo['HORAEXTRA_SABADO_1'] * 60) + $dadosVinculo['MINUTOS_EXTRA_SABADO_1']) / 60; 
                    $horaExtraSabado2 =  (($dadosVinculo['HORAEXTRA_SABADO_2'] * 60) + $dadosVinculo['MINUTOS_EXTRA_SABADO_2']) / 60; 
                    $horaExtraSabado3 =  (($dadosVinculo['HORAEXTRA_SABADO_3'] * 60) + $dadosVinculo['MINUTOS_EXTRA_SABADO_3']) / 60; 
                    $horaExtraSabado4 =  (($dadosVinculo['HORAEXTRA_SABADO_4'] * 60) + $dadosVinculo['MINUTOS_EXTRA_SABADO_4']) / 60; 
                    
                    $horaExtra1 =  (($dadosVinculo['HORAEXTRA_DIA_SEMANA_1'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_1']) / 60; 
                    $horaExtra2 =  (($dadosVinculo['HORAEXTRA_DIA_SEMANA_2'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_2']) / 60; 
                    $horaExtra3 =  (($dadosVinculo['HORAEXTRA_DIA_SEMANA_3'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_3']) / 60; 
                    $horaExtra4 =  (($dadosVinculo['HORAEXTRA_DIA_SEMANA_4'] * 60) + $dadosVinculo['MINUTOS_EXTRA_DIA_SEMANA_4']) / 60; 
                    
                    $horasDiurnas =  (($dadosVinculo['HORAS_DIURNAS_NAO_EX_SOMA'] * 60) + $dadosVinculo['MINUTOS_DIURNOS_NAO_EX_SOMA']) / 60; 
                    $horasNoturnas =  (($dadosVinculo['HORAS_NOTURNAS_NAO_EX_SOMA'] * 60) + $dadosVinculo['MINUTOS_NOTURNAS_NAO_EX_SOMA']) / 60; 
                    $horasFaltas =  (($dadosVinculo['HORAS_FALTAS_TOTAL'] * 60) + $dadosVinculo['MINUTOS_FALTA_TOTAL']) / 60; 
                    
                    //Dia de Semana e gerado 50%
                    if($horaExtra1){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_1;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtra1);
                    }
                    if($horaExtra2){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_1;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtra2);
                    }
                    if($horaExtra3){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_1;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtra3);
                    }
                    if($horaExtra4){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_1;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtra4);
                    }
                    
                     //Domingo E Feriado e Gerado 70%
                    if($horaExtraSabado1){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_5;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraSabado1);
                    }
                    if($horaExtraSabado2){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_5;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraSabado2);
                    }
                    if($horaExtraSabado3){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_5;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraSabado3);
                    }
                    if($horaExtraSabado4){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_5;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraSabado4);
                    }
                    
                    //Domingo E Feriado e Gerado 100%
                    if($horaExtraDomingoFeri1){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_11;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraDomingoFeri1);
                    }
                    if($horaExtraDomingoFeri2){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_11;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraDomingoFeri2);
                    }
                    if($horaExtraDomingoFeri3){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_11;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraDomingoFeri3);
                    }
                    if($horaExtraDomingoFeri4){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_HORAEXTRA_11;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horaExtraDomingoFeri4);
                    }
                    
                    if($horasFaltas){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_ATRASOS;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horasFaltas);
                    }
                    
                    if($horasNoturnas){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_ADIC_NOTURNO_1;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horasNoturnas);
                    }
                    
                    if($alteSalario->getIdtpsalario() == 'HORA' && ($horasDiurnas || $horasNoturnas)){
                        $nrEventom = FpatipoassoceveEntity::ASSOC_SALARIO_2;
                        $this->insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, 0, $horasDiurnas + $horasNoturnas);
                    }
                }
            }
        }
        
    }
    
    public function exportarTemplate($row){
        $nrestrutlegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL']: '';
        $nrestrutgeren = isset($row['NRESTRUTGEREN']) ? $row['NRESTRUTGEREN']: '';
        $nrestrutsind = isset($row['NRESTRUTSIND']) ? $row['NRESTRUTGEREN']: '';
        $nrtipohierarq = isset($row['NMTIPOHIERARQORG']) ? $row['NMTIPOHIERARQORG']: '';
        $nrestruthierarq = isset($row['NRESTRUTHIERARQ']) ? $row['NRESTRUTHIERARQ']: '';
        $eventos = $row['NREVENTOM'];
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();
        
        $cjEventos = $this->verificarInformacoes($nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrtipohierarq, $nrestruthierarq, $eventos, $nrorg, $nrestruthierarq);
        $objVinculosh = $this->LoteLancEventoRepository->getVinculos($nrestrutlegal, $nrestrutgeren, $nrestrutsind, $dtmescompetencia, $nrorg, $nrtipohierarq, $nrestruthierarq);
        $infos = $this->getInfoVinculos($objVinculosh, $dtmescompetencia, $nrorg, $nrtipohierarq);
        $file = new ManipulacaoExcel();
        foreach($infos as $info){
            $file->gravar($info, $cjEventos, $nrorg);
        }
        
    }
    
    private function verificarInformacoes( $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrtipohierarq, $nrestruthierarq, $eventos, $nrorg)
    {
        if (!empty($nrestrutlegal) || !empty($nrestrutgeren) || !empty($nrestrutsind) || !empty($nrtipohierarq) || !empty($nrestruthierarq)) {
            $this->verificaEstrutura($nrestrutlegal, $nrorg);
            $this->verificaEstrutura($nrestrutgeren, $nrorg);
            $this->verificaEstrutura($nrestrutsind, $nrorg);
            $cjEventos = $this->verificaEventos($eventos, $nrorg);
            if(!empty($nrestruthierarq) && empty($nrtipohierarq)){
                throw new \Exception(self::MSG_ESTRUTURA_HIERARQUIACA);
            }
        } else {
            throw new \Exception("Não há estrutura");
        }

        if ($cjEventos) {
            return $cjEventos;
        } else {
            return false;
        }
    }
    
    private function verificaEstrutura($nrestrutura, $nrorg) {
        if ($nrestrutura) {
            $object = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg,
                                                                                              'nrestruturam' => $nrestrutura));                                                  
        }else{
            return false;
        }

        if (!$object) {
            throw new \Exception("Não há estrutura");
        }else{
            return true;
        }
        
    }

    private function verificaEventos( $eventos)
    {
        if(!is_array($eventos))
            throw new \Exception('Nenhum evento selecionado.');
        $cjeventos = array();
        foreach ($eventos as $evento) {
            $cjeventos[] = array('CODIGO' => $evento['nreventom'],
                                 'EVENTO' => $evento['nmeventoh']);
        }
        return $cjeventos;
        
    }
    
    private function getInfoVinculos( $infoVinculos, $dtmescompetencia, $nrorg, $nrtipohierarq )
    {
        $info = array();
        $nrestrutgeren = -1;
        
        foreach ($infoVinculos as $infoVinculo) {
            if(!isset($infoVinculo['NRVINCULOH'])){
                $infoVinculo = $infoVinculos;
            }
            if(isset($infoVinculo['NRESTRUTURAM'])){ //Verifica se deve utilizar a estrutura hierárquica ou gerencial
                $nrestrutura = (int)$infoVinculo['NRESTRUTURAM'];
                if($nrestrutura > 0){
                    $nmhierarq = $this->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findOneBy(array('nrorg' => array($nrorg, 0), 'nrtipohierarqorg' => $nrtipohierarq));
                    $nmhierarq = $nmhierarq[0]->getNmtipohierarqorg();
                }else{
                    $nmhierarq = 'Hierarquia não cadastrada';
                }
            }else{
                $nrestrutura = $infoVinculo['NRESTRUTGEREN'];
                $nmhierarq   = 'Estrutura Gerencial';
            }
            if ($nrestrutgeren != $nrestrutura) {
                $nrestrutgeren = $nrestrutura;//offset invalido
                if (!$info[$nrestrutgeren]) {
                    $info[$nrestrutgeren] = array();
                }
            }

        
            //Objeto Vínculom
           
            $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrvinculom' => $infoVinculo['NRVINCULOM'], 'nrorg' => $infoVinculo['NRORG']));
            

            //Ferias e Situação Funcional
            if (!$this->getFerias($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia)) {
                $nmsitufunc = $this->getSitufunc($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia);
            } else {
                $nmsitufunc = self::NMFERIAS;
            }

            //Cargo
            $nmcargo = $this->getCargo($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia);

            //Construir Informação
            $info[$nrestrutgeren][] = array('ID' => $infoVinculo['NRVINCULOM'],
                'NOME' => $objVinculom->getNmvinculom(),
                'SITUFUNC' => $nmsitufunc,
                'ADMISSAO' => $objVinculom->getDtadmissaovinc()->format('d/m/Y'),
                'CARGO' => $nmcargo,
                'ESTRUTLEGAL' => $this->getNmestrutura($infoVinculo['NRESTRUTLEGAL'], $nrorg, $dtmescompetencia),
                'ESTRUTGEREN' => $nrestrutura > 0 ? $this->getNmestrutura($nrestrutura, $nrorg, $dtmescompetencia) : 'sem estrutura',
                'ESTRUTSINDI' => $this->getNmestrutura($infoVinculo['NRESTRUTSIND'],  $nrorg, $dtmescompetencia),
                'COMPETENCIA' => $dtmescompetencia,
                'NMESTRUTURA' => $nmhierarq,
                'NMGEREN'     => $this->getNmestrutura($infoVinculo['NRESTRUTGEREN'], $nrorg, $dtmescompetencia) );


        }
        

        return $info;
    }
    
    private function getFerias( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        
        $gozoFeriasRepository = new GozoFeriasRepository($this->entityManager->getConnection(), $this->entityManager);
        
        $result = $gozoFeriasRepository->retornaGozoFerias($nrvinculom, $nrorg);//aprovado
        
        if ($result) {
            return true;
        } else {
            return false;
        }
        
    }
    
    private function getSitufunc( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        
        $objAltesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtmescompetencia); //aprova

        if ($objAltesitufunc) {
            $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
            $nrsitufuncm = $objAltesitufunc->getNrsitufuncm();
            $objSitufunc = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia($nrorg, $nrorgPadrao, $nrsitufuncm, $dtmescompetencia);
        }

        if ($objSitufunc) {
            return $objSitufunc->getNmsitufunch();
        } else {
            return '';
        }
        
    }

    private function getCargo( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        $objAlteocup = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->retornaUltimaAlteracaoOcupacaoVinculo($nrorg, $dtmescompetencia, $nrvinculom);

        if ($objAlteocup) {
            $objOcupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistorico($objAlteocup->getNrocupacaom(), $nrorg,$dtmescompetencia);
        }

        if ($objOcupacao) {
            return $objOcupacao->getNmocupacaoh();
        } else {
            return false;
        }
    }
    
    private function getVinculom( $nrvinculom, $nrorg)
    {
        $objVinculom =  $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrvinculom' => $nrvinculom, 
                                                                                                              'nrorg' => $nrorg));

        if ($objVinculom) {
            return $objVinculom;
        } else {
            return false;
        }
        
    }

    private function getNmestrutura( $nrestruturam, $nrorg, $dtmescompetencia)
    {
        $objEstruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($nrorg, $nrestruturam, $dtmescompetencia);

        if ($objEstruturam) {
            return $objEstruturam->getNmrazsocestrut();
        } else {
            return '';
        }
    }
    
    
    public function importaArquivoPlanilha($row){
        try {
            $this->itensLote = [];
            $this->mensagens = [];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
            $mesCompetencia = $this->hcmParameters->getDtCompetencia();
            $this->dtmescompetenc = $mesCompetencia;
            $arquivo = $row['DSPATH'][0];
            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
            
            if (!$file->isFile()) {
                $this->mensagens[] = "Falha ao importar o arquivo.";
            }
            
            $this->entityManager->beginTransaction();
            $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
            $this->validaExtensao($caminhoNovo, $arquivo['name']);
            
            $info = new ManipulacaoExcel();
            $info = $info->preparaInformacao($caminhoNovo);
            $this->validaFormatacao($info, $arquivo['name']);
            $this->validaInformacao($info, $nrorg, $this->dtmescompetenc, $arquivo['name']);
            
            if(!$this->mensagens){
                
                $infoFinal = $this->preparaInfo($this->preparaInfoNome($row), $info, $nrorg, $this->dtmescompetenc);
                
                $this->insereLoteEvento($infoFinal['LOTE'], $info['CABECALHO']);
                $this->insereItemEvento($infoFinal['ITEM'], $arquivo);
                
                $this->entityManager->commit();
                return $this->mensagens;
            } else {
                return array_unique($this->mensagens);
                $this->entityManager->rollback();
            }
            

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function importaArquivoPlanilhaModelo2($row){
        try {
            $this->itensLote = [];
            $this->mensagens = [];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
            $mesCompetencia = $this->hcmParameters->getDtCompetencia();
            $this->dtmescompetenc = $mesCompetencia;
            $arquivo = $row['DSPATH'][0];
            $this->entityManager->beginTransaction();
            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory() . '/');
            if (!$file->isFile()) {
                throw new \Exception("Falha ao ler o arquivo.");
            }
            $caminho = $file->getPathname();

            $infos = \PHPExcel_IOFactory::load($caminho);
            $sheet = $this->verificaNomePlanilha($infos);
            
            if(!empty($sheet)){
                $colunas = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'];
                $mes = '';
                $valida = false;
                
                foreach($colunas as $index => $coluna){
                    $columnValue = $sheet->getCell($coluna.'3')->getCalculatedValue();
                    $comp = strtoupper(str_replace(['Ê', 'ê', ' ', '(', ')', ':'], ['E', 'e', '', '', '', ''], $columnValue));
                    if(!empty($columnValue)){
                       if($comp == 'MESCOMPETENCIA'){
                           $valida = true;
                       }else if($valida){
                           $mes = $columnValue;
                           break;
                       }
                    }
                }
                
                $dtmescompetenc = $this->retornaCompetencImportacao($mes);
                
                $indexColuna = 2;
                $eventos = array();
                $posiçãoArray = 0;
                while (!empty($colunas[$indexColuna])){
                    $value = $sheet->getCell($colunas[$indexColuna].'6')->getValue();
                    
                    if(substr($value,0,11) != 'REFLEXO DSR' && $value != 'A PAGAR' && $value != $evento[$posiçãoArray-1]['NREVENTOM'] && !empty($value)){
                        if(is_string($value) && substr($value, 0, 1) == '=' && empty($sheet->getCell($colunas[$indexColuna + 1].'6')->getValue())){
                            $value = str_replace(["$", "PROCV", "!N", ":O"], ["", "'PROCV'", "!N1", ":O100"], $value);
                            $value = \PHPExcel_Calculation::getInstance($sheet->getParent())->_calculateFormulaValue($value, $colunas[$indexColuna].'6', $sheet->getCell($colunas[$indexColuna].'6'));
                        }
                        $eventos[$posiçãoArray] = ['COLUNA'   => $colunas[$indexColuna].'6 ',
                                                  'CODIGO' => $value];
                                                  
                        $this->validaInformacaoEvento([$eventos[$posiçãoArray]], $nrorg, $sheet->getTitle());
                         
                        $eventos[$posiçãoArray]['COLUNA'] = $colunas[$indexColuna];
                    }
                    $indexColuna++;
                    $posiçãoArray++;
                }
                
                $this->validaExistenciaEvento($eventos, $sheet->getTitle());

                $linha = 9;
                $dados = array();
                $posiçãoArray = 0;
                $linhasVazias = 0;
                while ($linhasVazias <= 5 ){
                    if(!empty($sheet->getCell('A'.$linha)->getCalculatedValue())){
                        $vinculos = ['LINHA'  => 'A'.$linha,
                                     'CODIGO' => $sheet->getCell('A'.$linha)->getCalculatedValue()];
                        $this->validaFormatacaoFuncionario([$vinculos], $sheet->getTitle());
                        $this->validaInformacaoFuncionario([$vinculos], $nrorg, $sheet->getTitle());
                        
                        foreach($eventos as $evento){
                            
                            $dados[$posiçãoArray] = ['CODIGO'           => $vinculos['CODIGO'],
                                                     'EVENTO'           => $evento['CODIGO'],
                                                     'DTMESCOMPETENCIA' => DateUtil::getDataDeString($dtmescompetenc),
                                                     'VALOR'            => round($sheet->getCell($evento['COLUNA'].$linha)->getCalculatedValue(), 2),
                                                     'OBSERVACAO'       => ''];
                            $this->validaInformacaoValor([$dados[$posiçãoArray]], $sheet->getTitle());
                            $posiçãoArray++;
                        } 
                        $linha++;
                        $posiçãoArray++;
                    }else{
                        $linhasVazias ++;
                    }
                }
                $this->validaInformacaoInfo($dados[0], $nrorg, $this->dtmescompetenc, $sheet->getTitle());
                
                if(!$this->mensagens){
                    $info['INFO'] = $dados;
                    $infoFinal = $this->preparaInfo($this->preparaInfoNome($row), $info, $nrorg, $this->dtmescompetenc);
                    $this->insereLoteEvento($infoFinal['LOTE']);
                    $this->insereItemEvento($infoFinal['ITEM'], $sheet->getTitle());
                    
                    $this->entityManager->commit();
                    return $this->mensagens;
                } else {
                    return array_unique($this->mensagens);
                    $this->entityManager->rollback();
                }
            }else{
                throw new \Exception("Pagina 'Folha de Movimentação' ou 'Folha de Gratificação' não foi encontrada!");
            }

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    private function validaExtensao($caminho, $arquivo){
        $nome = preg_split("/(\\|\/|\.)/", $caminho);
        $extencao = strtoupper(end($nome));
        switch ($extencao) {
            case('XLS'):{
                return 1;
            }
            case('XLSX'):{
                return 2;
            }
            default:{
                $this->mensagens[] = "Extensão Inválida. (" . $arquivo . ")";
            }
        }
    }
    
    private function validaFormatacao($info, $arquivo){
        $this->validaFormatacaoFuncionario($info['FUNCIONARIO'], $arquivo);
        $this->validaFormatacaoEvento($info['EVENTO'], $arquivo);
        $this->validaFormatacaoInfos($info['INFO'], $arquivo);
    }
    
    private function validaFormatacaoFuncionario($funcs, $arquivo){
        if ($funcs) {
            foreach ($funcs as $func) {
                if (intval($func['CODIGO']) == 0) {
                    if($funcs['LINHA']){
                        $this->mensagens[] = "Funcionário Inválido inserido na linha " . $funcs['LINHA'] . ". (" . $arquivo . ")";
                    }
                }
            }
            return true;
        } else {
            $this->mensagens[] = "Não há vínculos. (" . $arquivo . ")";
        }
    }

    private function validaFormatacaoEvento($eventos, $arquivo){
        if ($eventos) {
            foreach ($eventos as $evento) {
                if (intval($evento['CODIGO']) == 0) {
                    $this->mensagens[] = "Evento Inválido inserido na coluna " . $eventos['COLUNA'] . ". (" . $arquivo . ")";
                }
            }
        } else {
            $this->mensagens[] = "Não há eventos. (" . $arquivo . ")";
        }
        return true;
    }
    
    private function validaFormatacaoInfos($infos, $arquivo){
        foreach ($infos as $info) {
            if(intval($info['CODIGO']) == 0){
                $this->mensagens[] = "Funcionário inválido inserido na linha " . $info['LINHA'] . ". (" . $arquivo . ")";
            }
            if(intval($info['EVENTO']) == 0){
                $this->mensagens[] = "Evento Inválido. (" . $arquivo . ")";
            }
            if(!($info['DTMESCOMPETENCIA'] instanceof \DateTime)){
                $this->mensagens[] = "Data de competência Inválida inserido na linha " . $info['LINHA'] . ". (" . $arquivo . ")";
            }
            if(($info['VALOR'] != 0) && (floatval($info['VALOR']) < 0) && (!is_null($info['VALOR']))){
                $this->mensagens[] = "Valor inválido inserido na linha " . $info['LINHA'] . ", no vinculo ". $info['CODIGO'] . " no evento " . $info['EVENTO'] . ". (" . $arquivo . ")";
            }
        }
        return true;
    }
    
    private function validaInformacao($info, $nrorg, $dtmescompetencia, $arquivo){
        $this->validaInformacaoFuncionario($info['FUNCIONARIO'], $nrorg, $arquivo);
        $this->validaInformacaoEvento($info['EVENTO'], $nrorg, $arquivo);
        $this->validaInformacaoInfo($info['CABECALHO'], $nrorg, $dtmescompetencia, $arquivo);
    }

    private function validaInformacaoFuncionario($funcs, $nrorg, $arquivo){
        foreach ($funcs as $func) {
            
            $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrorg,
                                                                                                   'nrvinculom' => $func['CODIGO']));

            if (!$vinculo){
                $this->mensagens[] = "Funcionário informado na linha " . $func['LINHA'] . " inexistente. (" . $arquivo . ")";
            } 
        }
    }

    private function validaInformacaoEvento($eventos, $nrorg, $arquivo){
        foreach ($eventos as $evento) {
            
            $objeto = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->findOneBy(array('nrorg' => array($nrorg, 0),
                                                                                                  'nreventom' => $evento['CODIGO']));

            if (!$objeto) {
                $this->mensagens[] = "Evento informado na coluna " . $evento['COLUNA'] . "Inválido. (" . $arquivo . ")";
            }
        }
    }

    private function validaExistenciaEvento($eventos, $arquivo){
        if(empty($eventos)){
            $this->mensagens[] = "Nenhum Evento foi encontrado. (" . $arquivo . ")";
        }
    }

    private function validaInformacaoInfo($infos, $nrorg, $dtmescompetencia, $arquivo){
        $competencia = substr($dtmescompetencia, 0, 10);
        if (!empty($infos['DTMESCOMPETENCIA']) && $competencia != $infos['DTMESCOMPETENCIA']->format('d/m/Y')) {
            $this->mensagens[] = "Data de competência incorreta. (" . $arquivo . ")";
        }
    }
    
    private function validaInformacaoValor($infos, $arquivo){
        foreach($infos as $info){
            if(($info['VALOR'] != 0) && (floatval($info['VALOR']) < 0) && (!is_null($info['VALOR']))){
                $this->mensagens[] = "Valor inválido inserido na linha " . $info['LINHA'] . ", no vinculo ". $info['CODIGO'] . " no evento " . $info['EVENTO'] . ". (" . $arquivo . ")";
            }
        }
    }
    
    private function preparaInfo($caminho, $info, $nrorg, $dtmescompetencia){
        $lotevento = $this->preparaInfoLoteEvento($caminho, $nrorg, $dtmescompetencia);
        $itensEvento = $this->preparaInfoItemEvento($info['INFO'], $nrorg, $lotevento->getNrlotelancevento());

        return array(
            'LOTE' => $lotevento,
            'ITEM' => $itensEvento
        );
    }
    
    private function preparaInfoLoteEvento($caminho, $nrorg, $dtmescompetencia){
        try {
            $nome = $caminho;
            $nrlotelacevento = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_LOTELANCEVENTO', 1, 12, true);
            
            $fpaLotelancevento = Repositories::FPA_LOTELANCEVENTO;

            $loteevento = new $fpaLotelancevento;

            $loteevento->setNrlotelancevento($nrlotelacevento);
            $loteevento->setDslotelancevento("IMPORTAÇÃO DE ARQUIVO $nome");
            $loteevento->setDtlotelancevento(DateUtil::getDataAtual());
            $loteevento->setDtcompetloteeve(DateUtil::getDataDeString($dtmescompetencia));
            $loteevento->setIdstatuslotelanc('PENDENTE');
            $loteevento->setDsobslotelanc('');
            $loteevento->setNrtipomoviment(1);
            $loteevento->setIdoriglotelanc('PLANILHA');
            $loteevento->setIdativo('S');
            $loteevento->setNrorginclusao($nrorg);
            $loteevento->setDtinclusao(DateUtil::getDataAtual());
            $loteevento->setCdoperinclusao($this->hcmParameters->getCdOperador());
            $loteevento->setNrorgultatu($nrorg);
            $loteevento->setDtultatu(DateUtil::getDataAtual());
            $loteevento->setCdoperultatu($this->hcmParameters->getCdOperador());
            $loteevento->setNrorg($nrorg);

            return $loteevento;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function preparaInfoItemEvento($infos, $nrorg, $nrlotelacevento){
        try {
            $itens = array();
            $nadaImportado = true;
            foreach ($infos as $info) {
                
                if (!is_null($info['VALOR']) && !($info['VALOR'] == 0) && is_numeric($info['VALOR'])) {
                    $codigo = NovoCodigo::geraCodigo($nrorg, 'FPA_ITEMLOTELANC', 1, 12, true);
                    
                    $fpaItemLotelanc = Repositories::FPA_ITEMLOTELANC;
                    
                    $item = new $fpaItemLotelanc;

                    $item->setNritlotelanc($codigo);
                    $item->setNrlotelancevento($nrlotelacevento);
                    $item->setNrvinculom($info['CODIGO']);
                    $item->setNreventom($info['EVENTO']);
                    $item->setIdativo('S');
                    $item->setNrorginclusao($nrorg);
                    $item->setDtinclusao(DateUtil::getDataAtual(false));
                    $item->setDtocorrencia(DateUtil::getDataAtual(false));
                    $item->setCdoperinclusao($this->hcmParameters->getCdOperador());
                    $item->setNrorgultatu($nrorg);
                    $item->setDtultatu(DateUtil::getDataAtual(false));
                    $item->setCdoperultatu($this->hcmParameters->getCdOperador());
                    $item->setNrorg($nrorg);
                    $item->setDsobservacao($info['OBSERVACAO']);
                    //salva observação 
                    
                    $ref = $this->entityManager->getRepository(Repositories::FPA_EVENTOH)->findOneBy(array('nreventom' => $item->getNreventom(), 'nrorg' => array($nrorg,0)), array('dtmescompetenc' => 'DESC'));
                    
                    if ($ref) {
                        if ($ref->getIdmediavalref() == 'REFERENCIA') {
                            $item->setVrreffolhalote($info['VALOR']);
                            $item->setVreventolote('');
                        } else {
                            $item->setVreventolote($info['VALOR']);
                            $item->setVrreffolhalote('');
                        }
                    }
                    
                    $itens[] = $item;
                    $nadaImportado = false;
                }
            }
            if($nadaImportado) {
                $this->mensagens[] = "Nenhum item foi importado porque não havia valor!";
            }
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function preparaInfoNome($row){
        $nome = $row['DSPATH'][0]['name'];
        return $nome;
    }
    
    private function retornaCompetencImportacao($mesPlanilha){
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $meses = ['01' => 'JANEIRO', 
                  '02' => 'FEVEREIRO',
                  '03' => 'MARCO',
                  '04' => 'ABRIL',
                  '05' => 'MAIO',
                  '06' => 'JUNHO',
                  '07' => 'JULHO',
                  '08' => 'AGOSTO',
                  '09' => 'SETEMBRO',
                  '10' => 'OUTUBRO',
                  '11' => 'NOVEMBRO',
                  '12' => 'DEZEMBRO'];
                  
        $mesPlanilha = strtoupper(str_replace(['ç'], ['c'], $mesPlanilha));

        foreach($meses as $numero => $mes){
            if($mesPlanilha == $mes){
                $dtmescompetenc = '01/'.$numero.'/'.substr($dtmescompetenc, 6, 4);
            }
        }
        
        return $dtmescompetenc;
    }
    
    private function verificaNomePlanilha($infos){
        $invalidChar = ['ã', 'Ã', 'ç', 'Ç', ' '];
        $validChar   = ['a', 'a', 'c', 'c', '' ];
        
        //a busca pela 'Folha de Movimentação' é priorizada em relação a 'Folha de Gratificação'
        foreach ($infos->getSheetNames() as $sheetNum => $sheetName) {
            $sheetTitle = strtoupper(str_replace($invalidChar, $validChar, $sheetName)); //Padronização do nome, sem char especial e espaços, texto maiusculo
            if($sheetTitle == 'FOLHADEMOVIMENTACAO')
                return $infos->getSheet($sheetNum);
        }
        
        foreach ($infos->getSheetNames() as $sheetNum => $sheetName) {
            $sheetTitle = strtoupper(str_replace($invalidChar, $validChar, $sheetName));
            if($sheetTitle == 'FOLHADEGRATIFICACAO')
                return $infos->getSheet($sheetNum);
        }
        
        return null;
    }
    
    private function insereLoteEvento($lote, $cabecalho = null){
        $nmstrutgeren = '';
        $nmstrutlegal = '';
        if(!empty($cabecalho)){
            $nmstrutgeren = $cabecalho['NMESTRUTGEREN'];
            $nmstrutlegal = $cabecalho['NMESTRUTLEGAL'];
        }
        $this->idoriglotelanc = $lote->getIdoriglotelanc();
        $this->insereLoteLancEvento($lote->getNrorg(), substr($lote->getDtcompetloteeve()->__toString(), 0, 10), $lote->getNrlotelancevento(), $lote->getDslotelancevento(), $lote->getIdstatuslotelanc(), $lote->getDsobslotelanc(), $lote->getNrtipomoviment(), $nmstrutlegal, $nmstrutgeren);
    }
    
    private function insereItemEvento($itens, $arquivo){
        $arrayItens = [];
        $nrorg = null;
        $nrlotelacevento = null;
        foreach ($itens as $dado){
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $dado->getNrvinculom(), "nrorg" => $dado->getNrorg()));
            $tipoVinculo = $vinculom->getNrtipovinculom();
            $bloq = $this->verificaEventoBloqueado($dado->getNrorg(), $dado->getNreventom(), $tipoVinculo);

            if ($bloq !== null){
                if ($bloq['NREVENTOMPARA'] === null) {
                    $this->msg[] = array('Operação bloqueada. O evento '.$dado->getNreventom().' não pode ser lançado para o vínculo "'.$vinculo.' - '.$vinculom->getNmvinculom(). '"!');
                    $disponibilidade = false;
                } else {
                    $msg[] = array('Operação bloqueada. O evento '.$dado->getNreventom().' não pode ser lançado para o vínculo "'.$vinculo.' - '.$vinculom->getNmvinculom().'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                    $disponibilidade = true;
                    $dado->setNreventom($bloq['NREVENTOMPARA']);
                }
            } else {
                $disponibilidade = true;
            }
    
            if ($disponibilidade === true) {
                $dados['NREVENTOM'] = $dado->getNreventom();
                $dados['VRREFFOLHALOTE'] = $dado->getVrreffolhalote();
                $dados['NRVINCULOM'] = $dado->getNrvinculom();
                $dados['VREVENTOLOTE'] = $dado->getVreventolote();
                $dados['NRLOTELANCEVENTO'] = $dado->getNrlotelancevento();
                $dados['DSOBSERVACAO'] = $dado->getDsobservacao();
                $dados['NRORG'] = $dado->getNrorg();
                $arrayItens[] = $dados;
                $nrorg = ($nrorg != null && $dados['NRORG'] != $nrorg) ? 'nrorgDiferente' : $dados['NRORG'];
                $nrlotelacevento = ($nrlotelacevento != null && $dados['NRLOTELANCEVENTO'] != $nrlotelacevento) ? 'nrlotelanceventoDiferente' : $dados['NRLOTELANCEVENTO'];
                
            }
        }
        if (count($msg) > 0) {
            foreach ($msg as $mensagem) {
                $this->mensagens[] = $mensagem;
            }
        }
        if($nrorg != 'nrorgDiferente' && $nrlotelacevento != 'nrlotelanceventoDiferente'){
            $this->insereItensLoteLancEvento($nrorg, $arrayItens, $nrlotelacevento);
            $this->entityManager->flush();
        } else {
            $this->mensagens[] = "Itens inconsistentes. (" . $arquivo . ")";
        }
    }
    
    public function importaArquivoTxt($row){
        $this->itensLote = [];
        $this->mensagens = [];
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $this->dtmescompetenc = $mesCompetencia;
        $arquivo = $row['DSPATH'][0];
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        
        if (!$file->isFile()) {
            throw new \Exception("Falha ao gravar o arquivo.");
        }
        
        $arquivo = file_get_contents($file->getPathname());
        $arquivo = str_replace("\r", '', $arquivo);
        $arquivo = explode("\n", $arquivo);
        
        foreach ($arquivo as $linha => $dados) {
            $this->processaLinha($linha, $dados) === false ? false : $salvarDados;
        }
        
        if(!$this->mensagens){
            
            $name = explode('.', $file->getClientOriginalName())[0];
            $dslotelanc = 'IMPORTAÇÃO TXT - '.$name;
            
            $this->idoriglotelanc = 'IMPORTAÇÃO TXT';
            
            $lote = $this->insereLoteLancEvento($nrorg, $mesCompetencia, null, $dslotelanc, 'PENDENTE', $dsobslote, 1);
            $nrLote = $lote->getNrlotelancevento();
            
            foreach ($this->itensLote as $itemLote) {
                $this->insereItemLote($itemLote['NRVINCULOM'], $nrorg, $nrLote, $itemLote['NREVENTOM'], $itemLote['VREVENTO'], 0, true);
            }
            
        }else{
            return $this->mensagens;
        }
        
    }
    
    
    public function processaLinha($linha, $dados){
        
        try {
            
            $dados = $this->_removeCaracteresEspeciais($dados);
            // Verifica tamanho da linha
            if(strlen($dados) == 0){
                return;
            }
            
            if (strlen($dados) != 28) {
                $this->mensagens[] = $this->tamLinhaInvalido($linha, strlen($dados));
                return false;
            }
            
            $nrestruturam = $this->getCodigoEmpresa($linha, $dados);
            $this->nrestruturam = $nrestruturam;
            $nrvinculom = $this->getCodigoFuncionario($linha, $dados);
            $nreventom = $this->getCodigoEvento($linha, $dados);
            $vrevento = $this->getValorEvento($linha, $dados);
            $dtmescompetenc = $this->getDataReferencia($linha, $dados);
            
            if (in_array(false, array($nrestruturam, $nrvinculom, $nreventom, $vrevento, $dtmescompetenc))) {
                return false;
            }
            
            $index = $this->getIndexItemLoteEvento($nrvinculom, $nreventom);
            
            if ($index === null) {
                // Salva dados da linha no array de itens
                $this->itensLote[] = array(
                    'NRESTRUTURAM' => $nrestruturam, 'NRVINCULOM' => $nrvinculom, 'NREVENTOM' => $nreventom, 'VREVENTO' => $vrevento, 'DTMESCOMPETENC' => $dtmescompetenc
                );
            } else {
                // Atualiza valor no array de itens
                $this->itensLote[$index]['VREVENTO'] += $vrevento;
            }
            
            return true;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    
    private function getValorEvento($linha, $dados)
    {
        $vrEvento = substr($dados,12,6);

        if (!is_numeric($vrEvento)) {
            $this->mensagens[] = $this->valorEventoInvalido($linha, $vrEvento);
            return false;
        }

        $vrEvento = floatval(substr_replace($vrEvento, '.', 4, 0));

        return $vrEvento;
    }

    private function getDataReferencia($linha, $dados)
    {
        $data = substr($dados,24,4);

        if (!is_numeric($data) || intval(substr($data,0,2)) < 1 || intval(substr($data,0,2)) > 12 || intval(substr($data,2,2)) < 1) {
            $this->mensagens[] = $this->dataRefenciaInvalida($linha, $data);
            return false;
        }

        $dtReferencia = DateUtil::getDataDeString($data, 'my', true);
        $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);

        if ($dtReferencia->format('m/Y') != $dtCompetencia->format('m/Y')) {
            $this->mensagens[] = $this->dataRefenciaDifCompetenc($linha, $data);
            return false;
        }

        return $dtReferencia->format(DateUtil::FORMATO_BRASILEIRO);
    }
    
    private function getIndexItemLoteEvento($nrvinculom, $nreventom)
    {
        foreach ($this->itensLote as $index=>$item) {
            if ($item['NRVINCULOM'] == $nrvinculom && $item['NREVENTOM'] == $nreventom) {
                return $index;
            }
        }
        
        return null;
    }
    
    public function getCodigoEmpresa($linha, $dados){
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $codEmpresa = substr($dados, 0, 3);
        
        $params = array('p_cdintestrutura'=> intval($codEmpresa), 'p_nrorg'=> $nrorg
        , 'p_tipoestrutcdintes' => $this->hcmParameters->getTipoEstruturaLegalOrg().intval($codEmpresa));
        
        $fpaEstruturam =  'Teknisa\Model\Geral\Entities\Estruturam';

$sql = <<<SQL
SELECT e FROM $fpaEstruturam e
WHERE e.nrorg = :p_nrorg 
AND (e.cdintestrutura = :p_cdintestrutura or 
e.cdintestrutura = :p_tipoestrutcdintes)
SQL;
        $result = $this->entityManager->createQuery($sql)->execute($params);
        
        if(!$result){
            $this->mensagens[] = $this->codEmpresaInvalido($linha, $codEmpresa);
            return false;
        }
        
        return $result[0]->getNrestruturam();
    }
    
    public function getCodigoFuncionario($linha, $dados)
    {
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $codFuncionario = substr($dados,3,6);
        
        $params = array('P_CDMATRICULA'=> $codFuncionario, 'P_NRORG'=>$nrorg, 'P_DTMESCOMPETENCIA' => $this->hcmParameters->getDtCompetencia());
        
        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        $sql = "SELECT V.* FROM GPE_VINCULOM V
                        WHERE V.NRORG = :P_NRORG 
                        AND LPAD(V.CDMATRICULA, 6, '0') = :P_CDMATRICULA 
                        AND V.NRVINCULOM = (SELECT VMM.NRVINCULOM FROM 
                                                GPE_VINCULOM VMM
                                                JOIN GPE_VINCULOH VHH ON 
                                                VHH.NRORG = VMM.NRORG 
                                                AND VHH.NRVINCULOM = VMM.NRVINCULOM 
                                                WHERE VMM.NRORG = :P_NRORG 
                                                AND LPAD(VMM.CDMATRICULA, 6, '0') = :P_CDMATRICULA 
                                            AND (VMM.DTRESCISAOVINC IS NULL OR VMM.DTRESCISAOVINC >= :P_DTMESCOMPETENCIA) AND ROWNUM <= 1)";
        
        $result = $db->fetchAll($sql, $params)[0];
        
        if(!$result){
            $this->mensagens[] = $this->codFuncionarioInvalido($linha, $codFuncionario);
            return false;
        }
        
        return $result['NRVINCULOM'];
        
    }

    public function getCodigoEvento($linha, $dados)
    {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $codEvento = substr($dados, 9,3);

        $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
        $sql = "SELECT *
                FROM MIGRAMDEPARA
                WHERE
                NMTABELADBF = 'EVENTO'
                AND NRORG = :P_NRORG
                AND NRDE = :P_NRDE";
                
        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRDE' => $codEvento
        );
        
        $migramdepara = $db->fetchAll($sql,$params)[0];
        
        if (!$migramdepara) {
            $this->mensagens[] = $this->codEventoInvalido($linha, $codEvento);
            return false;
        }

        return $migramdepara['NRPARA'];
    }
    
    private static function _removeCaracteresEspeciais($string){
        $string = preg_replace(self::REGEX_CARACTERES_ALFANUMERICOS, self::CARACTER_VAZIO, $string);
        $string = preg_replace(self::REGEX_QUEBRA_LINHA_CR_LF, self::CARACTER_VAZIO, $string);
        return $string;
    }
    
    public function insereItemLote($vinculo, $nrorg, $nrLote, $nrEventom, $vrLote, $vrRefLote, $verificaIdmedia = false){
        
        if(!$verificaIdmedia){
            $nrvinculom = $vinculo->getNrvinculom();
        }else{
            $nrvinculom = $vinculo;
        }
        
        $itensLote = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_ITEMLOTELANC)->findBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(),
        'nrlotelancevento' => $nrLote, 'nreventom' => $nrEventom, 'nrvinculom' => $nrvinculom));  
        
        if($itensLote){
            return;
        }
        
        if($verificaIdmedia){
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($nrEventom, $nrorg, 0, $this->dtmescompetenc);
            
            if($evento->historico){
                if ($evento->historico->getIdmediavalref() == 'REFERENCIA') {
                    $vrRefLote = $vrLote;
                    $vrLote = null;
                } else {
                    $vrRefLote = null;
                }
            }else{
                $vrRefLote = null;
                $vrLote = null;
            }
        }
        
        $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $vinculo, "nrorg" => $nrorg));
        $tipoVinculo = $vinculom->getNrtipovinculom();
        $bloq = $this->verificaEventoBloqueado($nrorg, $nrEventom, $tipoVinculo);
        
        if ($bloq !== null){
            if ($bloq['NREVENTOMPARA'] === null) {
                $this->msg[] = array('Operação bloqueada. O evento '.$nrEventom.' não pode ser lançado para o vínculo "'.$vinculo.' - '.$vinculom->getNmvinculom(). '"!');
                $disponibilidade = false;
            } else {
                $msg[] = array('Operação bloqueada. O evento '.$nrEventom.' não pode ser lançado para o vínculo "'.$vinculo.' - '.$vinculom->getNmvinculom().'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                $disponibilidade = true;
                $nrEventom = $bloq['NREVENTOMPARA'];
            }
        } else {
            $disponibilidade = true;
        }

        if ($disponibilidade === true) {
            $iteLote = new $this->itemLoteLanc;
            
            $iteLote->setVreventolote($vrLote);
            $iteLote->setVrreffolhalote($vrRefLote);
            $iteLote->setNrvinculom($nrvinculom);
            $iteLote->setNreventom($nrEventom);
            $iteLote->setNrlotelancevento($nrLote);
            $iteLote->setNrorg($nrorg);
            $iteLote->setIdativo('S');
            $iteLote->setDtinclusao(DateUtil::getDataAtual());
            $iteLote->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $iteLote->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $iteLote->setDtultatu(DateUtil::getDataAtual());
            $iteLote->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $iteLote->setCdoperultatu($this->hcmParameters->getCdoperador());
    
            $iteLote->setNritlotelanc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITEMLOTELANC', 1, 12, true));
            
            $this->entityManager->persist($iteLote);
            $this->entityManager->flush($iteLote);
        }
        if (count($msg) > 0) {
            $this->mensagens[] = $msg;

        }
    }
    
    public function verificaEventoBloqueado ($nrorg, $nrevento, $tipoVinculo) {
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrorg_padrao = $this->hcmParameters->getNrorgPadrao();
        return $this->bloqueioEventVincRepository->retornaEventosBloqueados($nrorg, $nrorg_padrao, $dtmescompetenc, $nrevento, $tipoVinculo);
    }
    
    public function horaToMin($x) {
    	if( $x != null) {
    	    $x = str_replace(":", '', $x);
    	    $h = substr($x, 0, 2)*60;
    		$m = substr($x, 2, 2); 
    		return $h + $m; 
    	} else {
    		return null;
    	}
    }
    
    public function minToHora($x){
        if($x != null){
            $h = floor($x / 60);
            if($h<10)
                $h='0'.$h;
            $m = $x % 60;
            if($m<10)
                $m='0'.$m;
            return $h.':'.$m.':00';
        } else {
            return false;
        }
    }
    
    public function horaToDecimal($x){
        return $this->horaToMin($x)/60;
    }
    
    public function percentFloat($x){
        return substr($x, 0, 2).'.'.substr($x, 2, 2);
    }
    
    public function funcionarioInvalido($funcionario)
    {
       return "O código do funcionário $funcionario está incorreto";
    }

    public function eventoInvalido($evento)
    {
        return "O código do funcionário $evento está incorreto";
    }

    public function valorInvalido($evento, $funcionario)
    {
        return "O valor informado para o evento $evento do vínculo $funcionario é inválido";
    }

    public function dataCompetenciaInvalido()
    {
        return "Data de Competência inválida";
    }

    public function extensaoInvalida()
    {
        return "Extensão de arquivo não aceita";
    }

    public function funcionarioInexistente($funcionario)
    {
        return "O funcionário $funcionario não existe ou seu código está incorreto";
    }

    public function dataCompetenciaIncorreta()
    {
        return "A Data de Competência do arquivo diferente da Data de Competência do sistema";
    }

    public function semEventos()
    {
        return "Não existem eventos para serem importados ou o template está incorrento";
    }

    public function eventoInexistente($codEvento)
    {
        return "Não existe o evento $codEvento";
    }

    public function infoInvalida()
    {
        return "Informações ou layout do arquivo inválidos";
    }

    public function lerArquivo()
    {
        return "Erro ao abrir ou ler o arquivo";
    }

    public function fazerDownload()
    {
        return "Erro ao tentar fazer download do arquivo";
    }

    public function semVinculos()
    {
        return "Não existem vínculos na planílha ou layout do arquivo é inválido";
    }

    public function tamLinhaInvalido($linha, $tamanho)
    {
        return "Linha $linha com tamanho inválido (Esperado:28, Real:$tamanho)";
    }

    public function codEmpresaInvalido($linha, $codEmpresa)
    {
        return "O código da empresa $codEmpresa está incorreto (Linha:$linha)";
    }

    public function codFuncionarioInvalido($linha, $codFuncionario)
    {
        return "O código do funcionário $codFuncionario está incorreto (Linha:$linha)";
    }

    public function codEventoInvalido($linha, $codEvento)
    {
        return "O código do evento $codEvento está incorreto (Linha:$linha)";
    }

    public function valorEventoInvalido($linha, $vrEvento)
    {
        return "O valor do evento $vrEvento está incorreto (Linha:$linha)";
    }

    public function dataRefenciaInvalida($linha, $data)
    {
        return "A data $data está incorreta (Linha:$linha)";
    }

    public function dataRefenciaDifCompetenc($linha, $data)
    {
        return "A data $data está diferente da data de competência (Linha:$linha)";
    }
}