<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AlteSalarioService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\Messages;
use HCM\Util\FPA;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class AlteSalarioController extends Crud {

    protected $dataSourceName = 'gpe_altesalario';
    protected $dataSourceNameGet = 'getAltesalario';
    protected $dataSourceNameGetSalFaixa = 'getSalarioFaixa';
    protected $nmArquivoTemplateCon = 'contaSalarial';


    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'EXISTE_VINCULOS', 'P_NRESTRUTLEGALOPER', 'P_NRESTRUTGERENOPER', 'P_NRTIPOHIERARQORG', 'P_NRESTRUTSUPERIORM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AlteSalarioService $alteSalarioService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->alteSalarioService = $alteSalarioService;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }
    
    public function baixarPlanilhaSalarial(DTO\Request\DataSet $request, DTO\Response $response) {
    try{
        $file = $this->alteSalarioService->baixarPlanilhaSalarial(
            $this->hcmParameters->getNrorgTrabalho(),
            $this->nmArquivoTemplateCon);
            

        if($file != null){
            $retorno = array('NMARQUIVO' => $file);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
        }
        
    } catch(\Exception $e) {
        $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilhaSalarial(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->alteSalarioService->importarPlanilhaSalarial($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAltesalario (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        
        $filterData->addCondition('P_NRESTRUTLEGALOPER', '=', FPA::getNrestrutlegalOper());
        $filterData->addCondition('P_NRESTRUTGERENOPER', '=', FPA::getNrestrutgerenOper());

        foreach ($filterData->getConditions() as $filter) { 
            $columns = ['NRTIPOHIERARQORG', 'NRESTRUTSUPERIORM'];

            if(in_array($filter['columnName'], $columns)) {
                $filterData->addCondition('P_'.$filter['columnName'], 'IN', implode($filter['value'], ','));
            }
        }

        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getSalarioFaixa (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGetSalFaixa);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setAltesalario (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $dtaltesalario = $row['DTALTESALARIO'];
                $nrocorrencia = $row['NROCORRENCIA'];
                $vrsalario = $row['VRSALARIO'];

                if (!isset($row['DSVALSALARIOEXTENSO'])) {
                    $row['DSVALSALARIOEXTENSO'] = '';
                }
                if (!isset($row['DTBASEDISSIDIO'])) {
                    $row['DTBASEDISSIDIO'] = '';
                }
                
                if(array_key_exists('VRSALARIOFAIXA', $row) && !empty($row['VRSALARIOFAIXA'])){
                    $row['VRSALARIO'] = $row['VRSALARIOFAIXA'];
                }
                
                if ($row['__is_new']) {
                    // Valida se já existe salario para a mesma data e mesma ocorrencia
                    $this->alteSalarioService->beforeSave($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia);
                }

                // Verifica se o salário informado é inferior ao salário mínimo do valor de referência
                // if ($this->alteSalarioService->valorMenorSalarioMinimo($vrsalario)) {
                //     $response->addMessage(new DTO\Response\Message(Messages::HCM00000_GPE_ALTESALARIO_VRSALARIO_MENOR_SALMINIMO, DTO\Response\Message::TYPE_MESSAGE));
                // }

                if (array_key_exists('NRALTESALARIO', $row)) {
                    if ($row['NRALTESALARIO'] === '') {
                        $row['NRALTESALARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true);
                    }
                } else {
                    $row['NRALTESALARIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true);
                }
                
                parent::save($request, $response);
                
                if ($row['__is_new']) {
                    // Verifica se o vínculo possui histórico para a competência
                    $this->alteSalarioService->afterInsert($nrorg, $nrvinculom);
                }
                // Atualiza histórico do vínculo
                $this->alteSalarioService->afterSave($nrorg, $nrvinculom);
                
                if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteAltesalario(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getNrEscalaVinculo (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $result = $this->alteSalarioService->getNrEscalaVinculo($request->getDataSet()->getRows()[0]['NRVINCULOM']);
            
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('escalaVinculo', array($result)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}