<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\AlteEscalaRepository;
use HCM\Geral\Repository\ValidacaoSistemaRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculom;
use HCM\Util\DateUtil;
use HCM\Util\Util;

class AlteEscalaService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, VinculoServices $vinculoServices, AlteEscalaRepository $alteEscalaRepository, ValidacaoSistemaRepository $validacaoSistemaRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->vinculoServices = $vinculoServices;
        $this->alteEscalaRepository = $alteEscalaRepository;
        $this->validacaoSistemaRepository = $validacaoSistemaRepository;
        
        $this->gpe_alteEscala = Repositories::GPE_ALTEESCALA;
    }
    
    public function nova($nrorg, $nrvinculom, $nrEscalaTrabalho, $dtIniEscala, $observacao = null,
                         $dtFimEscala = null, $nrhordiam = null, $nrMotivoEscala = null, $nrturno = null,$qthrlimitesephextra = null)
    {
        $alteEscala = new $this->gpe_alteEscala;
        $alteEscala->setNrvinculom($nrvinculom);
        $alteEscala->setNrorg($nrorg);
        $alteEscala->setNrescalatrabm($nrEscalaTrabalho);
        $alteEscala->setDtiniescala($dtIniEscala);
        $alteEscala->setNrmotivoescala($nrMotivoEscala);
        $alteEscala->setNrturno($nrturno);
        $alteEscala->setDtfimescala($dtFimEscala);
        $alteEscala->setNrhordiam($nrhordiam);
        $alteEscala->setDsobservacao($observacao);
        $alteEscala->setQthrlimitesephextra($qthrlimitesephextra);

        $alteEscala->setDtinclusao(DateUtil::getDataAtual());
        $alteEscala->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteEscala->setDtultatu(DateUtil::getDataAtual());
        $alteEscala->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteEscala->setNralteescala(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true));
        $this->entityManager->persist($alteEscala);

    }
    
    public function beforeSave($nrorg, $nrvinculom, $dtiniescala) {
        try {
            $this->validaExisteDataEscala($nrorg, $nrvinculom, $dtiniescala);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterInsert($nrorg, $nrvinculom, $competencCadastro) {
        try {
            $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom, $competencCadastro);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterSave($nrorg, $nrvinculom, $nrescalatrabm, $dtiniescala, $dtfimescala, $idusarcomp=false) {
        try {
            if($dtfimescala) $dtfimescala = DateUtil::getDataDeString(substr($dtfimescala, 0, 10), DateUtil::FORMATO_BRASILEIRO);

            $dataCompetencia = DateUtil::getDataDeString(substr(
                $this->hcmParameters->getDtCompetencia(),
                0,
                10),
                DateUtil::FORMATO_BRASILEIRO);
            $dataInicioEscala = DateUtil::getDataDeString(substr(
                $dtiniescala,
                0,
                10),
                DateUtil::FORMATO_BRASILEIRO);
            if ($dataInicioEscala > $dataCompetencia) {
                $dtmescompetenc = $dtiniescala;
            }
            if ($dataInicioEscala <= $dataCompetencia) {
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            }

            if($idusarcomp){
                if ($dataInicioEscala > $dataCompetencia) {
                    $alteescala = $this->alteEscalaRepository->retornaUltimaAlteEscalaPorDataInicio($nrorg, $nrvinculom, DateUtil::getUltimoDiaDoMes($dtiniescala)->format(DateUtil::FORMATO_BRASILEIRO));
                }
                if ($dataInicioEscala <= $dataCompetencia) {
                    $alteescala = $this->alteEscalaRepository->retornaUltimaAlteEscalaPorDataInicio($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes());
                }
                if(!empty($alteescala)) //Somente ajusta a escala de trabalho se existir uma
                    $nrescalatrabm = $alteescala[0]['NRESCALATRABM'];
            }

            if ($dataInicioEscala > $dataCompetencia) {
                $proxalteescala = $this->alteEscalaRepository->retornaProximaAlteEscala($nrorg, $nrvinculom, DateUtil::getUltimoDiaDoMes($dtiniescala)->format(DateUtil::FORMATO_BRASILEIRO));
            }
            if ($dataInicioEscala <= $dataCompetencia) {
                $proxalteescala = $this->alteEscalaRepository->retornaProximaAlteEscala($nrorg, $nrvinculom, $this->hcmParameters->getUltimoDiaMes());
            }

            if ($proxalteescala) {
                $proxalteescala = $proxalteescala[0];
                $dtiniproxescala = DateUtil::getDataDeString(substr($proxalteescala['DTINIESCALA'], 0, 10), DateUtil::FORMATO_BRASILEIRO);
                $dtiniproxescala->sub(new \DateInterval('P1D'));
                $dtfimescala = $dtiniproxescala;
            }
            
            $dtiniescala = DateUtil::getDataDeString(substr($dtiniescala, 0, 10), DateUtil::FORMATO_BRASILEIRO);
            $dtmescompetenc = DateUtil::getDataDeString(substr($dtmescompetenc, 0, 10), DateUtil::FORMATO_BRASILEIRO);

            if ($dataInicioEscala > $dataCompetencia) {
                $this->atualizaEscalaHistoricoVinculo($nrorg, $nrvinculom, $nrescalatrabm, $dtiniescala);
            }
            if ($dataInicioEscala <= $dataCompetencia) {
                $this->atualizaEscalaHistoricoVinculo($nrorg, $nrvinculom, $nrescalatrabm, $dtmescompetenc);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function afterDelete($nrorg, $nrvinculom, $dtiniescala)
    {
        $dataCompetencia = DateUtil::getDataDeString(substr(
            $this->hcmParameters->getDtCompetencia(),
            0,
            10),
            DateUtil::FORMATO_BRASILEIRO);
        $dataInicioEscala = DateUtil::getDataDeString(substr(
            $dtiniescala,
            0,
            10),
            DateUtil::FORMATO_BRASILEIRO);
        if ($dataInicioEscala <= $dataCompetencia) {
            $dtiniescala = $this->hcmParameters->getDtCompetencia();
        }

        $nrescalatrabm = null;
        $alteescala = $this->alteEscalaRepository->retornaUltimaAlteEscalaPorDataInicio($nrorg, $nrvinculom, $dtiniescala);
        if (!empty($alteescala)) {
            $nrescalatrabm = $alteescala[0]['NRESCALATRABM'];
        }

        $this->atualizaEscalaHistoricoVinculo($nrorg, $nrvinculom, $nrescalatrabm, $dtiniescala);
    }

    /**
     * Valida se já existe outra escala para o vínculo com a mesma data de início
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $dtiniescala
     * @throws \Exception
     */
    public function validaExisteDataEscala($nrorg, $nrvinculom, $dtiniescala) {
        try {
            $alteescalas = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)
                ->findBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'dtiniescala' => DateUtil::getDataDeString($dtiniescala, DateUtil::FORMATO_BRASILEIRO, true)));
            if (!empty($alteescalas)) {
                throw new \Exception(Messages::HCM00000_GER_ALTEESCALA_DATA_ALREADY_EXIST_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza número da escala no histórico atual do vínculo
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $nrescalatrabm
     * @throws \Exception
     */
    public function atualizaEscalaHistoricoVinculo($nrorg, $nrvinculom, $nrescalatrabm, $dtiniescala = null) {
        try {
            if(!$dtiniescala){
                $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia( $nrorg, $nrvinculom );
            } else {
                $dataCompetenc = $dtiniescala;
                $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia( $nrorg, $nrvinculom, $dataCompetenc );
            }
            
            $gpevinculoh->setNrescalatrabm($nrescalatrabm);
            $gpevinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gpevinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculoh->setDtultatu(DateUtil::getDataAtual());
            
            $this->entityManager->persist($gpevinculoh);
            $this->entityManager->flush($gpevinculoh);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function geraEscala($dataSet){
        
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $msgRetorno = array();
        $resultado = array();

        //Vinculos
        $vinculos = $this->validacaoSistemaRepository->retornaVinculos($dataSet);
        $this->entityManager->beginTransaction();
        foreach($vinculos as $vinculo){
            $nrvinculom = $vinculo->getNrvinculom();
            $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom);  
            $vinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg,  "dtmescompetenc" => $dtCompetencia));
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $vinculoh->getNrvinculom(), "nrorg" => $nrorg));
            $rescisaovinc = $vinculom->getDtrescisaovinc();
            $escalaTrabalho = $vinculoh->getNrescalatrabm();
            
            $dataIniEscala = DateUtil::getDataDeString($dataSet['DTINIESCALA'], DateUtil::FORMATO_BRASILEIRO, true);
            
            $ultAlteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->obterEscalaEntrePeriodo($dataIniEscala, $dataIniEscala, $dataIniEscala, $nrvinculom, $nrorg)[0] ?? [];
            $ultAlteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findOneBy(array("nrorg" => $nrorg, "nralteescala" => $ultAlteEscala['NRALTEESCALA'] ?? null));
            
            $existeEscalaNessaData = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findOneBy(array("nrvinculom" => $vinculoh->getNrvinculom(), "nrorg" => $nrorg, "dtiniescala" => $dataIniEscala));
            if($existeEscalaNessaData != null) {
                $msgRetorno[] = array(
                    "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                    "DSERRO" => "Escala não inserida para o vinculo porque ele já possui alteração de escala para o mesmo dia!"
                );
                continue;
            }

            //verifica se o vínculo selecionado possui data de rescisão maior do que a data inicial da escala, se for verdadeiro, a escala poderá ser adicionada
            if ($rescisaovinc >= DateUtil::getDataDeString($dataSet['DTINIESCALA'], DateUtil::FORMATO_BRASILEIRO, true) || !$rescisaovinc){

                if(empty($ultAlteEscala) || $ultAlteEscala->getNrescalatrabm() != $dataSet['NRESCALATRABM'] || $ultAlteEscala->getNrturno() != $dataSet['NRTURNO']){
                    
                    $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true);
                    $obj = new $this->gpe_alteEscala;
                    $obj->setNrvinculom($vinculoh->getNrvinculom());
                    $obj->setNrorg($nrorg);
                    if($dataSet['DTINIESCALA']){
                        $obj->setDtiniescala(DateUtil::getDataDeString($dataSet['DTINIESCALA'], DateUtil::FORMATO_BRASILEIRO, true));
                    }
                    if($dataSet['DTFIMESCALA']){
                        $obj->setDtfimescala(DateUtil::getDataDeString($dataSet['DTFIMESCALA'], DateUtil::FORMATO_BRASILEIRO, true));
                    }
                    $obj->setNrmotivoescala($dataSet['NRMOTIVOESCALA']);
                    $obj->setNrturno($dataSet['NRTURNO']);
                    $obj->setNrhordiam($dataSet['NRHORDIAM']);
                    $obj->setDsobservacao($dataSet['DSOBSERVACAO']);
                    $obj->setQthrlimitesephextra($dataSet['QTHRLIMITESEPHEXTRA']);
                    
                    $obj->setDtinclusao(DateUtil::getDataAtual());
                    $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                    $obj->setDtultatu(DateUtil::getDataAtual());
                    $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                     
                    $obj->setNralteescala($codigo);
                    $obj->setNrescalatrabm($dataSet['NRESCALATRABM']);
                    
                    if ($dataSet['ESCALAEXLUSIVA'] === 'S') {
                        if (empty($ultAlteEscala) || $ultAlteEscala->getNrescalatrabm() == null) {
                            $msgRetorno[] = array(
                                "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                                "DSERRO" => "O vínculo não possui nenhuma escala anteriormente adicionada!"
                            );
                        } else {
                            //clonagem e pesquisa da escala atual do vínculo
                            $escala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findOneBy(array("nrorg" => $nrorg, "nralteescala" => $ultAlteEscala->getNralteescala()));
                            if ($escala == null) {
                                $msgRetorno[] = array(
                                    "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                                    "DSERRO" => "O vínculo está cadastrado com uma escala inexistente ou inativa!"
                                );
                            } else {
                                $segundaEscala = clone $escala;
                                
                                //set de todos os campos que serão modificados
                                $codigo2 = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true);
                                $segundaEscala->setNralteescala($codigo2);
                                $preparedData = DateUtil::getDataDeString($dataSet['DTFIMESCALA'], DateUtil::FORMATO_BRASILEIRO, true);
                                if ($preparedData >= $segundaEscala->getDtiniescala()){
                                    $segundaEscala->setDtiniescala(DateUtil::adicionaIntervalo($preparedData));
                                        
                                    //set auditoria
                                    $segundaEscala->setDtinclusao(DateUtil::getDataAtual());
                                    $segundaEscala->setCdoperinclusao($this->hcmParameters->getCdoperador());
                                    $segundaEscala->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                                    $segundaEscala->setDtultatu(DateUtil::getDataAtual());
                                    $segundaEscala->setCdoperultatu($this->hcmParameters->getCdoperador());
                                    $segundaEscala->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                                    $this->entityManager->persist($obj);
                                    $this->entityManager->persist($segundaEscala);
                                    $this->entityManager->flush();
                                } else {
                                    $msgRetorno[] = array(
                                        "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                                        "DSERRO" => "A escala exclusiva precisa ter data fim preenchida e maior do que a data inicial da atual escala do vínculo!"
                                    );
                                }
                            }
                        }
                        
                    } else {
                        $vinculoh->setNrescalatrabm($dataSet['NRESCALATRABM']);
                        if ($escalaTrabalho !== null){
                            $preparedData = DateUtil::getDataDeString($dataSet['DTINIESCALA'], DateUtil::FORMATO_BRASILEIRO, true);
                            if (!empty($ultAlteEscala)){
                                $preparedData->sub( new \DateInterval('P1D') );
                                $ultAlteEscala->setDtfimescala($preparedData);
                            }
                        }
                        $this->entityManager->persist($obj);
                        $this->entityManager->flush();
                    }
                    
                    $this->atualizaEscalaHistoricoVinculo($nrorg, $vinculo->getNrvinculom(), $vinculoh->getNrescalatrabm());
                } else{
                    $msgRetorno[] = array(
                        "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                        "DSERRO" => "O vínculo já possui essa escala e turno como escala ativa!"
                    );
                }
            } else {
                $msgRetorno[] = array(
                    "NRVINCULOM" => $vinculoh->getNrvinculom(), 
                    "DSERRO" => "A escala possui data inicial posterior à rescisão do vínculo!"
                );
            }
        }
        $this->entityManager->commit();
        
        if(count($msgRetorno) !== 0){
            $resultado[0] = $msgRetorno;
            $resultado[1] = 0;
            return $resultado;
        }else{
            $resultado[1] = 1;
            return $resultado;
        }
        
    }
    
    public function alteraDataIniEscala($nrOrg, $nrVinculom, $antigaDtInicioEscala, $novaDtInicioEscala) {
        try{
            $antigaDtInicioEscala = Util::datadeString($antigaDtInicioEscala,true);
            $novaDtInicioEscala = Util::datadeString($novaDtInicioEscala,true);
            
            $listaAlteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findBy(array('nrorg' => $nrOrg, 'nrvinculom' => $nrVinculom, 'dtiniescala' => $antigaDtInicioEscala));
            
            if($listaAlteEscala){
                foreach($listaAlteEscala as $altEscala){
                    $altEscala->setDtiniescala($novaDtInicioEscala);
                    $this->entityManager->persist($altEscala);
                    $this->entityManager->flush();
                }
            }
            
        }catch (Exception $e){
            throw $e;
        }
    }
        
    public function alteraDataFimUltimaAlteracao(string $nrorg, string $nrvinculom, ?string $dtfimescala) {
        
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $ultimaAlteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->getAlteEscalaByCompetencia($nrvinculom, $nrorg, $dtcompetencia);
        $ultimaAlteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->findOneBy(
            ['nrorg' => $nrorg, 'nralteescala' => $ultimaAlteEscala['NRALTEESCALA']]
        );
        
        if(empty($ultimaAlteEscala)) {
            return;
        }
        
        $datafimEscala = empty($dtfimescala) ? null : DateUtil::getDataDeString($dtfimescala,DateUtil::FORMATO_BRASILEIRO,false);
        $ultimaAlteEscala->setDtfimescala($datafimEscala);
        $this->entityManager->persist($ultimaAlteEscala);
        $this->entityManager->flush();
    }

} 