<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class EnviarEmailService {

    //const SUBJECT_RECUPERAR_SENHA = 'Recuperação de Senha';
    const FOOTERCONTENT = 'Tem alguma dúvida? Quer nos dar sugestões? <b>Entre em contato conosco.</b>';

    private $dsemailoper;
    protected $entityManager;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    /**
     * Carrega template do body do email
     * @param $novasenha
     * @return string $template
     * @author leonardocarvalho
     */
    private function getTemplate($title, $dsemailoper, $nmoperador, $headerContent, $content, $footerContent, $footerTemplate = null, $nmestruturam, $ignoreContentDefault = false){
        if ($footerContent === null) {
            $footerContent = self::FOOTERCONTENT;
        }

        if (!$ignoreContentDefault) {
            $contentTemplate = "
                <h1 style='font-size: 30px; margin: 0 0 15px 0; color: #76c0b3;'> $headerContent </h1>
                <p style='margin: 0; padding: 0 0 12px 0; line-height: 17px; text-align: justify;'>
                    $content
                </p>
            ";
        } else {
            $contentTemplate = $content;
        }

        $logo_organizacao = $this->getUrlLogoCliente($nmestruturam);
        $logo_bottom = 'https://frequencia.pebbian.com/images/pebbian_logotipo.png';

        $template = <<<HTML
                    <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
                    <html>
                    <head>
                        <title>$title</title>
                    </head>
                    <body style="text-align: center; padding-top: 2%; padding-left: 2%; background-color: #FFF/*rgb(118, 192, 179)*/; margin: 0px; height: auto;">
                        <div style="width:672px; height:auto; box-shadow: 10px 10px 5px #78887d; margin-bottom: 20px;">
                            <table cellpadding="0" cellspacing="0" style="width: 672px; height: auto; margin: 0 auto;">
                                <tr style="background-color: #FFF">
                                    <td height="20">
                                        &nbsp;
                                    </td>
                                </tr>
                                <tr style="background-color: #FFF;">
                                    <td style="text-align: center;">
                                        <img width="174" height="56" src="$logo_organizacao" />
                                    </td>
                                </tr>
                                <!--<tr style="background-color: #FFF">
                                    <td height="35">
                                        &nbsp;
                                    </td>
                                </tr>-->
                                <tr style="background-color: #FFF; text-align: left; font: normal 14px Arial, Tahoma, sans-serif; color: #333;">
                                    <td>
                                        <table cellpadding="0" cellspacing="0">
                                            <tr>
                                                <td width="24"></td>
                                                <td height="100%">
                                                    $contentTemplate
                                                </td>
                                                <td width="24"></td>
                                            </tr>
                                        </table>
                                    </td>
                                </tr>
                                <!--<tr style="background-color: #FFF">
                                    <td>
                                        &nbsp;
                                    </td>
                                </tr>
                                <tr style="background-color: #1b9772; color: #FFF; font: normal 13px Arial, Tahoma, sans-serif;">
                                    <td height="17">
                                        &nbsp;
                                    </td>
                                </tr>
                                <tr style="background-color: #1b9772; color: #FFF; font: normal 13px Arial, Tahoma, sans-serif;">
                                    <td align="center">
                                        $footerContent
                                    </td>
                                </tr>
                                <tr style="background-color: #1b9772; color: #FFF; font: normal 13px Arial, Tahoma, sans-serif;">
                                    <td height="10">
                                        &nbsp;
                                    </td>
                                </tr>-->
                                <tr>
                                    <td>
                                        <table cellpadding="0" cellspacing="0">
                                            <tr style="font: normal 14px Arial, Tahoma, sans-serif; text-align: left;">
                                                <!--<td style="padding-right: 30px; padding-top: 10px; padding-left: 30px;">
                                                    <img height="40" src="$logo_bottom" />
                                                </td>-->
                                                <td valign="center" style="padding-left: 40px; padding-top: 20px; padding-bottom: 20px;">
                                                    $footerTemplate
                                                </td>
                                            </tr>
                                        </table>
                                    </td>
                                </tr>
                            </table>
                        </div>
                    </body>
                    </html>
HTML;

        return $template;

    }

    /**
     * Envia email para operador informando a ele sua nova senha.
     * @param string $email Email do operador que tera sua senha enviada.
     * @throws \Exception
     * @throws exception
     * @author leonardocarvalho
     */
    public function enviarEmailAlerta($title, $email, $nmoperador, $nmestruturam, $subject, $content, $headerContent, $footerContent, $footerTemplate, $nmfrom, $from = null, $ignoreContentDefault = false, $attachment = null){

        try {
            $this->dsemailoper = $email;

            // Valida se o email é existente
            self::validaEmailExistente();

            $content = self::getTemplate($title, $this->dsemailoper, $nmoperador, $headerContent, $content, $footerContent, $footerTemplate, $nmestruturam, $ignoreContentDefault);

            // Manda um email para operador notificando a mudança de senha
            //$return = new Mail();
            $mail = $this->hcmParameters->returnParamsSmtp();
            $return = $mail->newEmail($this->dsemailoper, $content, $subject, $nmoperador, $nmfrom, $from, $attachment);
            return $return;

        }
        catch(\Exception $e){
            throw new \Exception($e->getMessage(), $e->getCode());
        }

    }

    public function enviarEmailPersonalizado($templateName, $email, $subject, $nmdestinatario, $nmremetente, $nrestruturam, Array $content, $from, $attachment = null, $validaEmailTabelaOper = null) {
        try {
            if($validaEmailTabelaOper != 'ignoraTabelaOperador'){
                $this->dsemailoper = $email;
                // Valida se o email é existente
                self::validaEmailExistente();
            }
            
            if(!$email){
                throw new \Exception("Email do ".$nmdestinatario." não encontrado");
            }
            // seta logo estrutura
            $logoEstrutura = self::getUrlLogoByNrestruturam($nrestruturam);
            $content = array_merge($content, array('URL_LOGO_CLIENTE' => $logoEstrutura));
            
            switch ($templateName) {
                case 'TEKNISA':
                    $templateTeknisa = new \HCM\Util\Mail\Templates\Teknisa\TemplateTeknisa($this->entityManager, $this->hcmParameters);
                    $content = $templateTeknisa->setValuesTemplate($content);
                    $mail = $this->hcmParameters->returnParamsSmtp();
                    $return = $mail->newEmail($email, $content, $subject, $nmdestinatario, $nmremetente, null, $attachment, true);
                    return $return;
                    break;
                
                default:
                    # code...
                    break;
            }

        } catch(\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }     
    }

    public function validaEmailExistente(){

        if (is_numeric($this->dsemailoper)) {
            $this->retornaEmailByCdoperador();
        }
        $sql = "SELECT NMOPERADOR
                FROM OPERADOR
                WHERE (DSEMAILOPER = '$this->dsemailoper')";

        try {
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute();
            $result = $std->fetchAll();

            if (count($result[0]) > 0) {
                //$this->environment->setNmOperador($result[0]['NMOPERADOR']);
                return true;
            } else {
                throw new \Exception("Email do operador não encontrado.");
            }
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }

    }

    public function getUrlLogoCliente($nmestruturam) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nmestruturam = strtolower(current(str_word_count($nmestruturam, 2)));
        $logo_organizacao = "https://hcmmobile.pebbian.com/images/". $nrorg ."/". $nmestruturam ."-logotipo.png";

        return $logo_organizacao;
    }

    public function getUrlLogoByNrestruturam($nrestruturam) {        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();

        if (!empty($nrestruturam)) {
            $sql = "SELECT DSURLLOGOESTRUTURA
                      FROM ESTRUTURAM
                     WHERE NRORG        = :P_NRORG
                       AND NRESTRUTURAM = :P_NRESTRUTURAM";
                       
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute(array(
                "P_NRORG"        => $nrorg,
                "P_NRESTRUTURAM" => $nrestruturam
            ));

            $result = $query->fetchAll();

            if (count($result) > 0)
                return $result[0]['DSURLLOGOESTRUTURA'];
            else 
                return null;
        } else {
            return null;
        }
    }

}