<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query;
use Doctrine\ORM\QueryBuilder;
use Setforms2\Shared\ORM\DateTime;
use Zeedhi\Framework\HTTP\Kernel;
use Teknisa\Libs\Util\Environment;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\DateUtil;
use HCM\Util\NovoCodigo;
use Gestaopessoas\Entities\PtoPeriodoapuracao;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Gestaopessoas\Entity\GpeAlteescala;
use HCM\Gestaopessoas\Entity\GpeIntervaloshorario;
use HCM\Gestaopessoas\Entity\GpeVinculoh;
use HCM\Gestaopessoas\Entity\PtoProgocorrencia;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Util;
use HCM\Geral\Services\CalendarioService;
use HCM\Geral\Repository\AlteEscalaRepository;
use HCM\PTO\Repository\HorarioRepository;
use HCM\Geral\Repository\VinculoRepository;
use Teknisa\Model\Gestaopessoas\Entities\GpeMarcacaoponto;
use HCM\Util\MsgProcesso;

class MarcacaoPontoService {

    protected $entityManager;
    /** @var Environment */
    protected $environment;

    /** @var GpeMarcacaoponto */
    protected $marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco;

    /** @var MsgProcesso */
    protected $msgProcesso;
    
    // Constantes usadas para a determinação de intervalos
    const IDTIPOINTERVALO_TRABALHO          = 'TRABALHO';
    const LABEL_IDTIPOINTERVALO_TRABALHO    = 'Trabalho';
    const IDTIPOINTERVALO_AUSENCIA          = 'AUSENCIA';
    const LABEL_IDTIPOINTERVALO_AUSENCIA    = 'Ausência';
    
    // Tipos de Marcação (Coletores)
    const NRCOLETOR_MARCACAO_DIGITADA               = 24;
    const NRCOLETOR_MARCACAO_IMPORTADA_REP          = 26;
    const CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT     = self::NRCOLETOR_MARCACAO_DIGITADA;
    const IMPORTACAOREP_DEFAULT_NRCOLETOR_DEFAULT   = self::NRCOLETOR_MARCACAO_IMPORTADA_REP;
    
    // Tipo de dia de ocorrência das marcações
    const IDDIAOCORRENCIA_DIA_ANTERIOR          = 'DA';
    const IDDIAOCORRENCIA_DIA_CORRENTE          = 'DC';
    const IDDIAOCORRENCIA_DIA_SEGUINTE          = 'DS';
    const IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR    = 'FA';
    const IDDIAOCORRENCIA_FORCA_DIA_CORRENTE    = 'FC';
    const IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE    = 'FS';
    
    const IMPORTACAO_TEMPO_TOLERANCIA = '02:00';
    const IMPORTACAO_TEMPO_TOLERANCIA_ANTERIOR = self::IMPORTACAO_TEMPO_TOLERANCIA;
    const IMPORTACAO_TEMPO_TOLERANCIA_SEGUINTE = self::IMPORTACAO_TEMPO_TOLERANCIA;
    
    const IMPORTACAO_DEFAULT_DIA = self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
    const IMPORTACAO_DIA_CORRENTE = self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
    const IMPORTACAO_DIA_SEGUINTE = self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE;
    const IMPORTACAO_DIA_ANTERIOR = self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR;
    
    // Agrupações de dias de ocorrencia
    const IDDIAOCORRENCIA_ANTERIORES    = [self::IDDIAOCORRENCIA_DIA_ANTERIOR, self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR];
    const IDDIAOCORRENCIA_CORRENTES     = [self::IDDIAOCORRENCIA_DIA_CORRENTE, self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE];
    const IDDIAOCORRENCIA_SEGUINTES     = [self::IDDIAOCORRENCIA_DIA_SEGUINTE, self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE];
    const CADAUTOMARC_DEFAULT_IDDIAOCORRENCIA_DIA_CORRENTE = self::IDDIAOCORRENCIA_DIA_CORRENTE;
    
    // ID Enquadra Horário
    const IDENQUADRAHORARIO_DEFAULT_ONCREATION  = 'N';
    const MARCDESPREZADA_DEFAULT_ONCREATION     = 'N';

    // Controla marcacao
    const IDCTRLMARC_NORMAL = 'N';
    const IDCTRLMARC_AUTOMATICO = 'A';

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Environment $environment, Kernel $httpKernel, CalendarioService $calendarioService, AlteEscalaRepository $alteEscalaRepository, HorarioRepository $horarioRepository, VinculoRepository $vinculoRepository) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->httpKernel = $httpKernel;
        $this->hcmParameters = $hcmParameters;
        $this->alteEscalaRepository = $alteEscalaRepository;
        $this->horarioRepository = $horarioRepository;
        $this->vinculoRepository = $vinculoRepository;
        $this->gpeVinculomEntity = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->gpeMarcacaoponto = Repositories::GPE_MARCACAOPONTO;
        $this->ptoProgOcorrencia = Repositories::PTO_PROGOCORRENCIA;
        $this->calendarioService = $calendarioService;
        $this->msgProcesso = MsgProcesso::getInstance();
    }
    
    public function getMarcacoesPonto($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT 
                TO_CHAR(S.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS SAIDA,
                TO_CHAR(E.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS ENTRADA,
                E.NRMARCACAOPONTO AS NRMARCACAOPONTO,
                E.IDENTRADASAIDA,
                TO_CHAR(S.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') || ' - ' || TO_CHAR(E.DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI') AS DESCRIPTION,
                E.NRORG
            FROM
                (SELECT MCP.*,
                    CASE
                        WHEN MOD(ROWNUM, 2) = 0 THEN 'S'
                        ELSE 'E'
                    END AS IDENTRADASAIDA,
                    ROW_NUMBER() OVER(PARTITION BY NRORG ORDER BY DTHRAJUSTADA) AS NRORDER_INTERV
                    FROM (
                        SELECT MP.*
                        FROM GPE_MARCACAOPONTO MP
                        WHERE MP.NRVINCULOM = :P_NRVINCULOM
                        AND MP.NRORG = :NRORG
                        AND MP.NRCOLETOR NOT IN(
                            (SELECT PORG.NRCOLETORMARCAUTOM FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCGERSIST FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCPROJETADA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCVIRADIA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG)
                        )
                        AND (
                            ((MP.IDDIAOCORRENCIA IS NULL OR MP.IDDIAOCORRENCIA IN ('FC', 'DC')) AND TO_CHAR(MP.DTHRAJUSTADA, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO) OR
                            ((MP.IDDIAOCORRENCIA IN ('FA', 'DA')) AND TO_CHAR(MP.DTHRAJUSTADA - 1, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO) OR
                            ((MP.IDDIAOCORRENCIA IN ('FS', 'DS')) AND TO_CHAR(MP.DTHRAJUSTADA + 1, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO)
                        )
                        ORDER BY MP.DTHRAJUSTADA
                    ) MCP
                ) E,
                (SELECT MCP.*,
                    CASE
                        WHEN MOD(ROWNUM, 2) = 0 THEN 'S'
                        ELSE 'E'
                    END AS IDENTRADASAIDA,
                    ((ROW_NUMBER() OVER(PARTITION BY NRORG ORDER BY DTHRAJUSTADA)) - 1) AS NRORDER_INTERV
                    FROM (
                        SELECT MP.*
                        FROM GPE_MARCACAOPONTO MP
                        WHERE MP.NRVINCULOM = :P_NRVINCULOM
                        AND MP.NRORG = :NRORG
                        AND MP.NRCOLETOR NOT IN(
                            (SELECT PORG.NRCOLETORMARCAUTOM FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCGERSIST FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCPROJETADA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG), 
                            (SELECT PORG.NRCOLETORMARCVIRADIA FROM GPE_PARAMORG PORG WHERE NRORG = :NRORG)
                        )
                        AND (
                            ((MP.IDDIAOCORRENCIA IS NULL OR MP.IDDIAOCORRENCIA IN ('FC', 'DC')) AND TO_CHAR(MP.DTHRAJUSTADA, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO) OR
                            ((MP.IDDIAOCORRENCIA IN ('FA', 'DA')) AND TO_CHAR(MP.DTHRAJUSTADA - 1, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO) OR
                            ((MP.IDDIAOCORRENCIA IN ('FS', 'DS')) AND TO_CHAR(MP.DTHRAJUSTADA + 1, 'DD/MM/YYYY') = :P_DTAPURACAOPONTO)
                        )
                        ORDER BY MP.DTHRAJUSTADA
                    ) MCP
                ) S
            WHERE E.NRORG = S.NRORG
            AND E.IDENTRADASAIDA = 'S'
            AND E.NRORDER_INTERV = S.NRORDER_INTERV
        ");
        $query->execute(
            array(
                "NRORG"           => $parameters["P_NRORG"],
                "P_NRVINCULOM"      => $parameters["P_NRVINCULOM"],
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );
        $marcacaoPontos = $query->fetchAll();
        
        return $marcacaoPontos;

    }

    private function verificaIdentificaIntervalo($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT PROG.*
              FROM PTO_PROGOCORRENCIA PROG
              JOIN GPE_MARCACAOPONTO MARC
                ON MARC.NRMARCACAOPONTO = PROG.NRMARCACAOPONTO
               AND MARC.NRVINCULOM      = PROG.NRVINCULOM
               AND MARC.NRORG           = PROG.NRORG
             WHERE PROG.NRORG                 = :P_NRORG
               AND PROG.NRVINCULOM            = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA IN (20, 26)
               AND PROG.DTINICIOPROGOCOR      = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO       = :P_NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $this->environment->getNrOrgTrab(),
                "P_NRVINCULOM"      => $parameters["NRVINCULOM"],
                "P_DTMARCACAOPONTO" => $parameters["DTMARCACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["NRMARCACAOPONTO"]
            )
        );

        $ocorrencias = $query->fetchAll();

        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    private function verificaAbonarSaida($parameters) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT
                *
              FROM PTO_PROGOCORRENCIA PROG
              LEFT JOIN PTO_MOTIVOOCORFREQ MOT
                ON MOT.NRMOTIVOOCORFREQ = PROG.NRMOTIVOOCORFREQ
             WHERE PROG.NRORG                = :P_NRORG
               AND PROG.NRVINCULOM           = :P_NRVINCULOM
               AND PROG.NRTIPOPROGOCORRENCIA = 17
               AND PROG.DTINICIOPROGOCOR     = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.DTFIMPROGOCOR        = TO_DATE(:P_DTMARCACAOPONTO, 'DD/MM/YYYY')
               AND PROG.NRMARCACAOPONTO      = :P_NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $this->environment->getNrOrgTrab(),
                "P_NRVINCULOM"      => $parameters["NRVINCULOM"],
                "P_DTMARCACAOPONTO" => $parameters["DTMARCACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["NRMARCACAOPONTO"]
            )
        );

        $ocorrencias = $query->fetchAll();

        if (isset($ocorrencias[0])) {
            return $ocorrencias[0];
        } else {
            return null;
        }
    }

    public function getDadosHorarioVinculo($nrorg, $nrvinculom, $dtapuracaoponto, $dtmescompetenc = null) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT HT.NRHORDIAM, HD.IDTIPOHORARIO, IH.NRINTERVALOHORARIO, HH.NMHORDIAH, AE.NRESCALATRABM, EH.QTHRESCALATRABH AS QTJORNADADIARIA
		  FROM (SELECT NRORG, MAX(NRTURNO) NRTURNO, NRESCALATRABM
					 FROM GPE_ALTEESCALA
					WHERE NRORG = :P_NRORG
					  AND NRVINCULOM = :P_NRVINCULOM
					  --AND TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
                      AND DTINIESCALA = (SELECT MAX(DTINIESCALA)
                                 FROM GPE_ALTEESCALA ALT
                                WHERE ALT.NRORG        = GPE_ALTEESCALA.NRORG
                                  AND ALT.NRVINCULOM   = GPE_ALTEESCALA.NRVINCULOM
                                  AND ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
					GROUP BY NRORG, NRESCALATRABM) AE,
				 (SELECT NRORG, NRTURNO, TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')-DTDATABASE AS NRDIASENTRE
					 FROM GPE_TURNO
					WHERE NRORG = :P_NRORG) TU,
				 (SELECT NRORG, NRTURNO, NRHORDIAM,
							ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					ORDER BY NRORG, NRTURNO, NRSEQHORARIO) HT,
				 (SELECT NRORG, NRTURNO, COUNT(*) AS QTHORARIOS
					 FROM GPE_HORARIOTURNO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRTURNO) QH,
				 (SELECT NRORG, NRHORDIAM, MAX(DTMESCOMPETENC) AS DTMESCOMPETENC, NMHORDIAH
					 FROM GPE_HORDIAH
					WHERE NRORG = :P_NRORG
					  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
					GROUP BY NRORG, NRHORDIAM, NMHORDIAH) HH,
				 (SELECT NRORG, NRHORDIAM, MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO
					 FROM GPE_INTERVALOSHORARIO
					WHERE NRORG = :P_NRORG
					GROUP BY NRORG, NRHORDIAM) IV, GPE_INTERVALOSHORARIO IH, GPE_HORDIAH HD, GPE_ESCALATRABH EH
		 WHERE AE.NRORG = TU.NRORG
			AND AE.NRTURNO = TU.NRTURNO
			AND AE.NRORG = HT.NRORG
			AND AE.NRTURNO = HT.NRTURNO
			AND HT.NRORG = HH.NRORG
			AND HT.NRHORDIAM = HH.NRHORDIAM
			AND HD.NRORG = HH.NRORG
			AND HD.NRHORDIAM = HH.NRHORDIAM
			AND HD.DTMESCOMPETENC = HH.DTMESCOMPETENC
			AND IV.NRORG(+) = HH.NRORG
			AND IV.NRHORDIAM(+) = HH.NRHORDIAM
			AND IH.NRORG(+) = IV.NRORG
			AND IH.NRHORDIAM(+) = IV.NRHORDIAM
			AND IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO
			AND AE.NRORG = QH.NRORG
			AND AE.NRTURNO = QH.NRTURNO
			AND HT.NRSEQHR-1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS)
      AND EH.NRESCALATRABM = AE.NRESCALATRABM
      AND EH.NRORG = AE.NRORG
      AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                 FROM GPE_ESCALATRABH
                                WHERE NRORG = EH.NRORG
                                  AND NRESCALATRABM = AE.NRESCALATRABM
                                  AND DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
        ");

        $query->execute(
            array(
                "P_NRORG"            => $nrorg,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_DTMESCOMPETENCIA" => is_null($dtmescompetenc) ? DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO) : $dtmescompetenc,
                "P_DTAPURACAOPONTO"  => $dtapuracaoponto,
            )
        );
        $dadosHorario = $query->fetchAll();

        if (isset($dadosHorario[0])) {
            return $dadosHorario[0];
        } else {
            return false;
        }
    }

    public function deleteMarcacoesPonto($rows) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        foreach($rows as $row){

            $nrmarcacaoponto_entrada = $row['NRMARCACAOPONTO'];
            $nrmarcacaoponto_saida   = $row['NRMARCACAOPONTO_SAIDA'];

            $query = "DELETE FROM
                        PTO_PROGOCORRENCIA
                      WHERE NRORG             = $nrorg
                        AND NRVINCULOM        = $nrvinculom
                        AND NRMARCACAOPONTO  IN ($nrmarcacaoponto_entrada, $nrmarcacaoponto_saida)";

            $result_ocorrencia = $this->entityManager->getConnection()->exec($query);

            $query_marc = "DELETE FROM
                      GPE_MARCACAOPONTO
                    WHERE NRORG      = $nrorg
                      AND NRVINCULOM = $nrvinculom
                      AND NRMARCACAOPONTO IN ($nrmarcacaoponto_entrada, $nrmarcacaoponto_saida)";

            $result_marcacao = $this->entityManager->getConnection()->exec($query_marc);

        }

        //if ($result_marcacao > 0)
            return array('DELETE' => true);
        /*else
            return array('DELETE' => false);*/

    }

    public function ausenciaAntesJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              NRMARCACAOPONTO,
              TO_CHAR(DTHRMARCACAOPONTO, 'HH24:MI') DTHRMARCACAOPONTO,
              (SELECT
                  SUBSTR(HRENTRADA, 1, 2) || ':' || SUBSTR(HRENTRADA, 3, 2) HRENTRADA
                 FROM GPE_INTERVALOSHORARIO
                WHERE NRORG = :P_NRORG
                  AND NRHORDIAM = :P_NRHORDIAM
                  AND HRENTRADA = (SELECT MIN(HRENTRADA)
                                     FROM GPE_INTERVALOSHORARIO INTER
                                    WHERE INTER.NRORG = GPE_INTERVALOSHORARIO.NRORG
                                      AND INTER.NRHORDIAM = GPE_INTERVALOSHORARIO.NRHORDIAM)) HRBASEENTRADA
             FROM GPE_MARCACAOPONTO
            WHERE NRORG      = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND NRCOLETOR  = 24
              AND DTHRMARCACAOPONTO = (SELECT MIN(MARC.DTHRMARCACAOPONTO)
                                         FROM GPE_MARCACAOPONTO MARC
                                        WHERE MARC.NRORG      = GPE_MARCACAOPONTO.NRORG
                                          AND MARC.NRVINCULOM = GPE_MARCACAOPONTO.NRVINCULOM
                                          AND MARC.NRCOLETOR  = GPE_MARCACAOPONTO.NRCOLETOR
                                          AND MARC.IDDIAOCORRENCIA IN ('FC', 'DC')
                                          AND MARC.DSJUSTIFICATIVA IS NULL
                                          AND TRUNC(MARC.DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRHORDIAM"       => $parameters["P_NRHORDIAM"]
            )
        );

        $ausenciaAntesJornada = $query->fetchAll();

        if (isset($ausenciaAntesJornada[0])) {
            return $ausenciaAntesJornada[0];
        } else {
            return null;
        }
    }

    public function ausenciaDepoisJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              MAX(NRMARCACAOPONTO) NRMARCACAOPONTO,
              TO_CHAR(DTHRMARCACAOPONTO, 'HH24:MI') DTHRMARCACAOPONTO,
              (SELECT
                  SUBSTR(HRSAIDA, 1, 2) || ':' || SUBSTR(HRSAIDA, 3, 2) HRSAIDA
                 FROM GPE_INTERVALOSHORARIO
                WHERE NRORG = :P_NRORG
                  AND NRHORDIAM = :P_NRHORDIAM
                  AND HRSAIDA   = (SELECT MAX(HRSAIDA)
                                     FROM GPE_INTERVALOSHORARIO INTER
                                    WHERE INTER.NRORG = GPE_INTERVALOSHORARIO.NRORG
                                      AND INTER.NRHORDIAM = GPE_INTERVALOSHORARIO.NRHORDIAM)) HRBASESAIDA
             FROM GPE_MARCACAOPONTO
            WHERE NRORG      = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULOM
              AND NRCOLETOR  = 24
              AND DTHRMARCACAOPONTO = (SELECT MAX(MARC.DTHRMARCACAOPONTO)
                                         FROM GPE_MARCACAOPONTO MARC
                                        WHERE MARC.NRORG      = GPE_MARCACAOPONTO.NRORG
                                          AND MARC.NRVINCULOM = GPE_MARCACAOPONTO.NRVINCULOM
                                          AND MARC.NRCOLETOR  = GPE_MARCACAOPONTO.NRCOLETOR
                                          AND MARC.IDDIAOCORRENCIA IN ('FC', 'DC')
                                          AND MARC.DSJUSTIFICATIVA IS NULL
                                          AND TRUNC(MARC.DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY'))
            GROUP BY DTHRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRHORDIAM"       => $parameters["P_NRHORDIAM"]
            )
        );

        $ausenciaDepoisJornada = $query->fetchAll();

        if (isset($ausenciaDepoisJornada[0])) {
            return $ausenciaDepoisJornada[0];
        } else {
            return null;
        }
    }

    public function getNrmarcacaoPontoSaidaAusenciaAntesJornada($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              MAX(NRMARCACAOPONTO) NRMARCACAOPONTO_SAID
             FROM GPE_MARCACAOPONTO
            WHERE NRORG               = :P_NRORG
              AND NRVINCULOM          = :P_NRVINCULOM
              AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND DSJUSTIFICATIVA IS NOT NULL
              AND NRCOLETOR = 24
              AND ROWNUM <= 2
            ORDER BY DTHRMARCACAOPONTO, NRMARCACAOPONTO
        ");

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );

        $nrMarcacaoSaidaAntesJornada = $query->fetchAll();

        if (isset($nrMarcacaoSaidaAntesJornada[0])) {
            return $nrMarcacaoSaidaAntesJornada[0];
        } else {
            return null;
        }
    }

    public function setIdfixaentsaidMarcacaoPonto($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND IDFIXAENTSAID            = :P_IDFIXAENTSAID
        ");

        $return = $sql->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_IDFIXAENTSAID"   => $parameters["P_IDFIXAENTSAID"]
            )
        );

        $query = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = :P_IDFIXAENTSAID
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRMARCACAOPONTO          = :P_NRMARCACAOPONTO
        ");

        $result_update = $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"],
                "P_NRMARCACAOPONTO" => $parameters["P_NRMARCACAOPONTO"],
                "P_IDFIXAENTSAID"   => $parameters["P_IDFIXAENTSAID"]
            )
        );

        if ($result_update)
            return array('UPDATE' => true);
        else
            return array('UPDATE' => false);
    }

    public function importaDigitacaoAutomatica($parameters) {
        $nrorg           = $this->environment->getNrOrg();
        $nrvinculom      = $this->environment->getNrvinculom();
        $dtapuracaoponto = $parameters["P_DTAPURACAOPONTO"];
        $nrinscricaoparc = $parameters["P_NRINSCRICAOPARC"];

        $connProducaoTeknisa = oci_connect('PRODUCAO_CONSULTA', 'tecnisa', '192.168.122.6/PDBORCL');

        // retorna horário base para validações de 5 minutos na entrada
        $horariobase = $this->getHorarioBase($nrorg, $parameters["P_NRHORDIAM"]);
        $dtHoraBaseEntrada = $dtapuracaoponto. ' ' .$horariobase['HRENTRADA_NOVA'];
        $horaBaseEntrada   = $horariobase['HRENTRADA_NOVA'];

        /*$query = "
            SELECT
                FU.CDFUNC,
                FU.NMFUNC,
                FU.NRCPFFUNC,
                FU.USR_NRVINCULO_SAAS,
                FU.USR_NRORG_SAAS,
                TO_CHAR(DT.DTDATA, 'DD/MM/YYYY')      DTDATA,
                TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI') DTBATIDAPTO
              FROM DATA DT
              JOIN FUNCIONARIO FU
                ON FU.NRORG     = 1
               AND FU.CDEMPRESA = '01'
               AND (FU.USR_NRVINCULO_SAAS IS NOT NULL OR FU.USR_NRVINCULO_SAAS IS NULL)
               AND (FU.USR_NRORG_SAAS     IS NOT NULL OR FU.USR_NRORG_SAAS     IS NULL)
              JOIN BATIDAPONTO PONTO
                ON PONTO.NRORG              = FU.NRORG
               AND PONTO.CDFUNC             = FU.CDFUNC
               AND TRUNC(PONTO.DTBATIDAPTO) = DT.DTDATA
              WHERE DT.DTDATA BETWEEN TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY') AND TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY')
                AND (FU.USR_NRVINCULO_SAAS = '$nrvinculom'
                     OR FU.NRCPFFUNC = '$nrinscricaoparc')
              ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO
        ";*/

        // LEONARDOCARVALHO - 16/02/2016 - Nova query tratando tolerância de 5 minutos na entrada
        $query = "
            SELECT
              RES.CDFUNC,
              RES.NMFUNC,
              RES.NRCPFFUNC,
              RES.USR_NRVINCULO_SAAS,
              RES.DTDATA,
              RES.DTBATIDAPTO_REAL,
              RES.DTBATIDAPTO,
              RES.DTBATIDAPTO_SAIDA
            FROM (SELECT
                    SEG.*,
                    CASE
                      WHEN TO_DATE(SEG.DTBATIDAPTO_REAL, 'HH24:MI') < TO_DATE('$horaBaseEntrada', 'HH24:MI') THEN 'AN'
                      WHEN TO_DATE(SEG.DTBATIDAPTO_REAL, 'HH24:MI') > TO_DATE('$horaBaseEntrada', 'HH24:MI') + 0.003472 THEN 'AP'
                      ELSE 'DE'
                    END FLAG1,
                    CASE
                      WHEN TO_DATE(SEG.DTBATIDAPTO_SAIDA, 'HH24:MI') < TO_DATE('$horaBaseEntrada', 'HH24:MI') THEN 'AN'
                      WHEN TO_DATE(SEG.DTBATIDAPTO_SAIDA, 'HH24:MI') > TO_DATE('$horaBaseEntrada', 'HH24:MI') + 0.003472 THEN 'AP'
                      ELSE 'DE'
                    END FLAG2
                  FROM(SELECT
                          PRI.*,
                          LEAD (PRI.DTBATIDAPTO_REAL, 1) OVER (ORDER BY PRI.USR_NRVINCULO_SAAS, PRI.DTBATIDAPTO) DTBATIDAPTO_SAIDA,
                          ROWNUM LIN
                        FROM (SELECT
                                CEN.*
                              FROM(SELECT
                                     FU.CDFUNC,
                                     FU.NMFUNC,
                                     FU.NRCPFFUNC,
                                     FU.USR_NRVINCULO_SAAS,
                                     FU.USR_NRORG_SAAS,
                                     PONTO.IDSENTBATIDA,
                                     TO_CHAR(DT.DTDATA, 'DD/MM/YYYY')      DTDATA,
                                     TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI') DTBATIDAPTO_REAL,
                                     DECODE(
                                       TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI') - PONTO.DTBATIDAPTO,
                                       ABS(TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI') - PONTO.DTBATIDAPTO),
                                       TO_CHAR(TO_DATE('$dtHoraBaseEntrada', 'DD/MM/YYYY HH24:MI'), 'HH24:MI'),
                                       TO_CHAR(PONTO.DTBATIDAPTO, 'HH24:MI')) DTBATIDAPTO,
                                     ROW_NUMBER() OVER (ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO) L
                                   FROM DATA DT
                                   JOIN FUNCIONARIO FU
                                     ON FU.NRORG     = 1
                                    AND FU.CDEMPRESA = '01'
                                    AND (FU.USR_NRVINCULO_SAAS IS NOT NULL OR FU.USR_NRVINCULO_SAAS IS NULL)
                                    AND (FU.USR_NRORG_SAAS     IS NOT NULL OR FU.USR_NRORG_SAAS     IS NULL)
                                   JOIN BATIDAPONTO PONTO
                                     ON PONTO.NRORG              = FU.NRORG
                                    AND PONTO.CDFUNC             = FU.CDFUNC
                                    AND TRUNC(PONTO.DTBATIDAPTO) = DT.DTDATA
                                   WHERE DT.DTDATA BETWEEN TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY') AND TO_DATE('$dtapuracaoponto', 'DD/MM/YYYY')
                                     AND (FU.USR_NRVINCULO_SAAS = '$nrvinculom'
                                          OR FU.NRCPFFUNC = '$nrinscricaoparc')
                                   ORDER BY FU.USR_NRVINCULO_SAAS, PONTO.DTBATIDAPTO
                                   ) CEN
                                   WHERE CEN.L != 1 OR CEN.IDSENTBATIDA != 2 /* Necessário para ignorar se o primeiro registro for uma saída */
                             ) PRI
                  ) SEG
                  WHERE MOD(SEG.LIN, 2) = 1
            ) RES
            WHERE ( RES.FLAG1 != 'AN' OR RES.FLAG2 != 'DE' ) AND ( RES.FLAG1 != 'AN' OR RES.FLAG2 != 'AN' )
              AND RES.DTBATIDAPTO_SAIDA IS NOT NULL
        ";

        $executeProducaoTeknisa = oci_parse($connProducaoTeknisa, $query);
        oci_execute($executeProducaoTeknisa);
        oci_fetch_all($executeProducaoTeknisa, $marcacoesCatraca, null, null, OCI_FETCHSTATEMENT_BY_ROW);

        $resultEnt = false;
        $resultSai = false;
        if (count($marcacoesCatraca) > 0) {

            // se total de marcações forem impar, retira a última marcação para formar sempre intervalos - NOVA QUERY
            /*if (count($marcacoesCatraca) % 2 != 0) {
                array_pop($marcacoesCatraca);
            }*/

            $dataset = array();
            foreach ($marcacoesCatraca as $marcacaoCatraca) {
                if ($marcacaoCatraca["USR_NRVINCULO_SAAS"] == $nrvinculom ||
                    $marcacaoCatraca["NRCPFFUNC"] == $nrinscricaoparc
                ) {

                    $dataset["idenquadrahorario"] = 'N';
                    $dataset["idmarcdesprezada"] = 'N';
                    $dataset["iddiaocorrencia"] = 'FC';
                    $dataset["dsjustificativa"] = '';
                    $dataset["nrorginclusao"] = '';
                    $dataset["nrcoletor"] = '24';
                    $dataset["idativo"] = 'S';
                    $dataset["dtinclusao"] = DateUtil::getDataAtual();
                    $dataset["cdoperinclusao"] = '';

                    // insere marcação de entrada
                    $dataset["dthrmarcacaoponto"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO"] . ":00";
                    $dataset["dthrajustada"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO"] . ":00";

                    $resultEnt = $this->insereMarcacoesPontoManual($dataset);

                    // insere marcação de saída
                    $dataset["dthrmarcacaoponto"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO_SAIDA"] . ":00";
                    $dataset["dthrajustada"] = $marcacaoCatraca["DTDATA"] . " " . $marcacaoCatraca["DTBATIDAPTO_SAIDA"] . ":00";

                    $resultSai = $this->insereMarcacoesPontoManual($dataset);
                }
            }
        }

        if ($resultEnt && $resultSai)
            return true;
        else
            return false;

    }

    public function getHorarioBase($nrorg, $nrhordiam) {

        $query = $this->entityManager->getConnection()->prepare("
            SELECT
              TO_CHAR(TO_DATE(HRENTRADA, 'HH24MI') - 0.003472, 'HH24:MI') AS HRENTRADA_NOVA,
              SUBSTR(SUBSTR(HRENTRADA, 1, LENGTH(TRIM(HRENTRADA)) -2) || ':' || SUBSTR(HRENTRADA, -2), 1, 10) AS HRENTRADA,
              (SELECT SUBSTR(SUBSTR(HRSAIDA, 1, LENGTH(TRIM(HRSAIDA)) -2) || ':' || SUBSTR(HRSAIDA, -2), 1, 10) AS HRSAIDA
                 FROM GPE_INTERVALOSHORARIO IH
                WHERE IH.NRORG     = :P_NRORG
                  AND IH.NRHORDIAM = :P_NRHORDIAM
                  AND IH.NRSEQINTERVHORARIO = (SELECT MAX(NRSEQINTERVHORARIO)
                                                 FROM GPE_INTERVALOSHORARIO
                                                WHERE NRORG     = IH.NRORG
                                                  AND NRHORDIAM = IH.NRHORDIAM)) HRSAIDA
              FROM GPE_INTERVALOSHORARIO IH
             WHERE IH.NRORG     = :P_NRORG
               AND IH.NRHORDIAM = :P_NRHORDIAM
               AND IH.NRSEQINTERVHORARIO = (SELECT MIN(NRSEQINTERVHORARIO)
                                              FROM GPE_INTERVALOSHORARIO
                                             WHERE NRORG     = IH.NRORG
                                               AND NRHORDIAM = IH.NRHORDIAM)
        ");

        $query->execute(
            array(
                "P_NRORG"     => $nrorg,
                "P_NRHORDIAM" => $nrhordiam
            )
        );

        $horariobase = $query->fetchAll();

        if (count($horariobase) > 0) {
            return $horariobase[0];
        } else {
            return null;
        }
    }

    public function updateMarcacaoEntradaSaida($marcacoes) {
        $parameters = array();
        $parameters["P_DTAPURACAOPONTO"] = $marcacoes[0]["P_DTAPURACAOPONTO"];
        $parameters["P_NRMARCACAOPONTO"] = $marcacoes[0]["P_NRMARCACAOPONTO"];
        $parameters["P_IDFIXAENTSAID"]   = $marcacoes[0]["P_IDFIXAENTSAID"];
        $this->setIdfixaentsaidMarcacaoPonto($parameters);
    }

    public function reverterEntradaSaida($parameters) {
        $nrorg      = $this->environment->getNrOrg();
        $nrvinculom = $this->environment->getNrvinculom();

        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
        ");

        $return = $sql->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_DTAPURACAOPONTO" => $parameters["P_DTAPURACAOPONTO"]
            )
        );

        if ($return)
            return array('UPDATE' => true);
        else
            return array('UPDATE' => false);
    }
    
    public function getMarcacoesVinculoMesCompetencia($nrOrg, $dtIniStr, $dtFimStr){
        $query = $this->entityManager->getConnection()->prepare(
                 "SELECT NRVINCULOM, 
                         TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY HH24:MI') DTHRMARCACAOPONTO, 
                         IDNSRREP
                    FROM GPE_MARCACAOPONTO
                   WHERE NRORG = :P_NRORG
                     AND DTHRMARCACAOPONTO BETWEEN TO_DATE(:P_INI, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:P_FIM, 'DD/MM/YYYY HH24:MI:SS')"
        );
        
        $query->execute(
            array(
                "P_NRORG" => $nrOrg,
                "P_INI"   => $dtIniStr,
                "P_FIM"   => $dtFimStr
            )
        );
        
        $retorno = $query->fetchAll();
        return $retorno;
        
    } 
    
    public function importaArquivoParaMarcacao($row){
        set_time_limit(0);
        $nrorg             = $row['NRORG'];
        $nrorg_padrao      = $row['NRORG_PADRAO'];
        $mesCompetencia    = $row['MESCOMPETENCIA'];
        $periodo           = $row['NRPERIODOAPURACAO'];
        $arquivo           = $row['DSPATH'][0];
        $dtinicialapuracao = !empty($row['DTINICIALAPURACAOFILTRO'])    ? $row['DTINICIALAPURACAOFILTRO'] . ' 00:00:00' : null;
        $dtfimapuracao     = !empty($row['DTFIMAPURACAOFILTRO'])        ? $row['DTFIMAPURACAOFILTRO'] . ' 23:59:00'     : null;
        $filtroTipoVinculo = $row["NRTIPOVINCULOM"] ?? null;
        
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/");
        if (!$file->isFile()) {
            throw new \Exception("Falha ao gravar o arquivo.");
        }
        
        $result =  $this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoIntervalo($nrorg, $periodo)[0];
        if (!$result) {
            throw new \Exception("Periodo não encontrado.");
        }
        
        if($result){
            $dtIniStr             = $result['DTINICIALAPURACAO'].' 00:00:00';   // Adiquiri as datas de escopo e ajusta a hora limite do dia
            $dtFimStr             = $result['DTFINALAPURACAO'].' 23:59:59';
            $dtIniObj             = DateUtil::getDataDeString($dtIniStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);  // Converte as datas de escopo para objeto
            $dtFimObj             = DateUtil::getDataDeString($dtFimStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);  // Converte as datas de escopo para objeto
            $tempoDescanco = $result['HRTEMPODESCANSODIARIO'];
            
            $dtinicialapuracaoObj = !empty($dtinicialapuracao)  ? DateUtil::getDataDeString($dtinicialapuracao, DateUtil::FORMATO_BRASILEIRO_DATAHORA)  : $dtIniObj;
            $dtfimapuracaoObj     = !empty($dtfimapuracao)      ? DateUtil::getDataDeString($dtfimapuracao, DateUtil::FORMATO_BRASILEIRO_DATAHORA)      : $dtFimObj;
        }
        
        $arquivo = Util::filterREPFile($file->getPathname(), $dtIniObj, $dtFimObj);
        
        $registros = array();
        $pisVinculoNaoEcontrado = '';
        
        foreach ($arquivo as $linha) {
            if($linha){
                $tpReg = substr($linha, 9, 1); // Tipo de registro 1: Cabeçalho, Tipo de registro 2: Inclusão ou Alteração da empresa no REP, Tipo de registro 3: Marcação de ponto
                if($tpReg == '3') {  // Somente são feitas buscas e adições de registros do tipo 3
                    $nsr = substr($linha, 0, 9);  // Número Sequencial de Registro
                    $data = substr($linha, 10,  8);
                    $hora = substr($linha, 18,  4);
                    $pis = substr($linha, 22, 12);
                    $dtMarcacaoStr = substr($data, 0, 2) . '/' . substr($data, 2, 2) . '/' . substr($data, 4, 4)  //Monta a data de marcação completa (em string). Parte da data (string)
                        . ' ' . substr($hora, 0, 2) . ':' . substr($hora, 2, 2) . ':00';                        //Parte da hora (string)
                    $dtMarcacaoObj = DateUtil::getDataDeString($dtMarcacaoStr, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    $vinculo = $this->gpeVinculomEntity->retornaVinculosAtivosMesCompetencia($nrorg, $mesCompetencia, null, null, null, false, null, true, false, null, null, null,  null, intval($pis));
                    
                    if(count($vinculo) >= 1){
                        $vinculo = $vinculo[0];

                        $tipoVinculo = $vinculo->getNrtipovinculom();

                        $tipoVinculoNotEqualFilter = (isset($filtroTipoVinculo) && $filtroTipoVinculo != "") && ($tipoVinculo != $filtroTipoVinculo);

                        if ($tipoVinculoNotEqualFilter) {
                            continue;
                        }

                        if(DateUtil::getIntervaloEmSegundos($dtinicialapuracaoObj, $dtMarcacaoObj) >= 0 && DateUtil::getIntervaloEmSegundos($dtMarcacaoObj, $dtfimapuracaoObj) > 0) {
                            $registros[$vinculo->getNrvinculom()][] = array(
                                'mar' => $dtMarcacaoObj,
                                'nsr' => $nsr
                            );
                        }
                    } else{
                        $pisVinculoNaoEcontrado .= ''.$pis.',';
                    }
                }
            }
        }
        
        if($pisVinculoNaoEcontrado){
            $mensagem['erro'] = 'Vínculo não encontrado para o(s) pis: '.$pisVinculoNaoEcontrado.'.'; 
        }
        
        $marcacoes = $this->getMarcacoesVinculoMesCompetencia($nrorg, $dtIniStr, $dtFimStr);
        
        $contVinc = 0;
        $contMarc = 0;
        foreach($registros as $vinculo => $registro){
            $marcAnterior  = null;
            $diaAnterior   = null;
            $idDia         = self::IMPORTACAO_DEFAULT_DIA;
            $idDiaAnterior = null;
            
            foreach($registro as $valor){
                 $quebraIteracao = false;
                
                if (isset($marcacoes)) {
                    foreach($marcacoes as $marcacao) {
                        if ($marcacao["NRVINCULOM"] == $vinculo) {
                            if ($marcacao["IDNSRREP"] == $valor['nsr'] && $marcacao["DTHRMARCACAOPONTO"] == $valor['mar']->format('d/m/Y H:i')) {
                                $quebraIteracao = true;
                            }
                        }
                    }
                }
                
                if ($quebraIteracao == true) {
                    continue;
                }
                
                $diaAtual = $valor['mar']->format('d/m/Y');
                $diaSubtraido = DateUtil::subtraiIntervalo($valor['mar'], 1)->format('d/m/Y');

                if(isset($marcAnterior)){
                    // Condição para tornar a primeira marcação do dia como 'FA' caso a diferença dessa pra anterior seja inferior ao tempo de descanso (param. org.) e não seja no mesmo dia da anterior
                    $dif = $valor['mar']->diff($marcAnterior);
                    $dif = ($dif->s / 3600) + ($dif->i / 60) + $dif->h + ($dif->d * 24) + ($dif->m * 720) + ($dif->y * 8760);
                    if($dif > $tempoDescanco) {
                        $idDia = self::IMPORTACAO_DIA_CORRENTE;
                    } else {
                        if($diaAtual == $diaAnterior) {
                            $idDia = $idDiaAnterior;
                        } else {
                            $idDia = self::IMPORTACAO_DIA_ANTERIOR;
                        }
                    }
                }
                
                $idDia = $this->getIdDiaMarcImportada($vinculo, $valor['mar']);
                
                $marcAnterior  = $valor['mar'];
                $diaAnterior   = $marcAnterior->format('d/m/Y');
                $idDiaAnterior = $idDia;
                $nrMarcacaoPonto = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_MARCACAOPONTO', 1, 12, true);
                $dtAtual = DateUtil::getDataAtual();
                
                $obj = new $this->gpeMarcacaoponto;  
                $obj->setNrvinculom($vinculo);                     
                $obj->setNrmarcacaoponto($nrMarcacaoPonto);    
                $obj->setNrorg($nrorg);
                $obj->setDthrmarcacaoponto($valor['mar']);
                $obj->setDthrajustada($valor['mar']);
                $obj->setIdnsrrep($valor['nsr']);
                $obj->setNrcoletor(self::IMPORTACAOREP_DEFAULT_NRCOLETOR_DEFAULT);
                $obj->setIddiaocorrencia($idDia);
                $obj->setIdenquadrahorario('N');
                $obj->setIdmarcdesprezada('N');
                $obj->setIdativo('S');
                $obj->setDtinclusao($dtAtual);
                $this->entityManager->persist($obj);
                $this->entityManager->flush($obj);
                
                $index = $idDia == 'FA' ? DateUtil::subtraiIntervalo($valor['mar'], 1)->format('d/m/Y') : $valor['mar']->format('d/m/Y');
                $horarioMarcacoes[$index][] = ['dthrmarcacaoponto' => $valor['mar'], 'nrmarcacaoponto' => $nrMarcacaoPonto, 'iddiaocorrencia' => $idDia];

                $contMarc++;                                    
            }

            if (isset($horarioMarcacoes)) {
                $intervalosDescanso = [];
                // Percorre as marcações inseridas para detectar intervalos
                foreach ($horarioMarcacoes as $marcacoesDia) {
                    $quantidadeMarcacoes = count($marcacoesDia);
                    $maiorTempoIntervaloDescanso = [];
                    
                    // Busca índices que estão relacionados com intervalos, considerando um array ordenado de marcações.
                    // Ex: 1° Marcação = Entrada; 2° Marcação = Saída; 3° Marcação = Entrada; 4° Marcação = Saída. O índice de um intervalo seria 1 (2° Marcação, primeira saída).
                    // Ver mais detalhes na descrição da função
                    $possiveisIntervalosDescanso = $this->getIndexesPossiveisDescanso($quantidadeMarcacoes);
                    foreach ($possiveisIntervalosDescanso as $index) {
                        $currentIntervalo = DateUtil::getIntervaloEmSegundos($marcacoesDia[$index]['dthrmarcacaoponto'], $marcacoesDia[$index+1]['dthrmarcacaoponto']);
                        
                        // Atualiza o maior intervalo
                        if ($currentIntervalo > ($maiorTempoIntervaloDescanso['tempo'] ?? 0)) {
                            $maiorTempoIntervaloDescanso = [
                                'tempo'             => $currentIntervalo,
                                'inicio'            => $marcacoesDia[$index]['dthrmarcacaoponto'],
                                'fim'               => $marcacoesDia[$index+1]['dthrmarcacaoponto'],
                                'nrmarcacaoponto'   => $marcacoesDia[$index]['nrmarcacaoponto'],
                                'data'              => $marcacoesDia[0]['dthrmarcacaoponto'],
                            ];
                        }
                    }
                    
                    // Guarda o maior intervalo para que a ocorrência seja inserida posteriormente
                    if (!empty($maiorTempoIntervaloDescanso)) {
                        $intervalosDescanso[] = $maiorTempoIntervaloDescanso;
                    }
                }
                
                $alteEscala = $this->alteEscalaRepository->retornaUltimaAlteEscalaPorDataInicio($this->hcmParameters->getNrorgTrabalho(), $vinculo, DateUtil::getPrimeiroDiaDoMes($valor['mar'])->format('d/m/Y'));
                if (empty($alteEscala)) {
                    continue;
                }
                $alteEscala = $alteEscala[0];
                
                $horaDia =  $this->entityManager->getRepository(Repositories::GPE_INTERVALOSHORARIO)->findBy(array("nrorg" => $nrorg, "nrhordiam" => $alteEscala['NRHORDIAM']));
                $nrintervalo = null;
                foreach($horaDia as $horario) {
                    if ($horario->getHrprojetarintervalo() !== null) {
                        $nrintervalo = $horario->getNrintervalohorario();
                        break;
                    }
                }
                
                // array para definir o id do dia da semana para a tabela
                $diaSemana = array('1' => '0100000',
                                   '2' => '0010000',
                                   '3' => '0001000',
                                   '4' => '0000100',
                                   '5' => '0000010',
                                   '6' => '0000001',
                                   '7' => '1000000');
                                   
                // percorre todos os intervalos de almoço salvos e salva uma ocorrência para cada um deles
                foreach ($intervalosDescanso as $intervalo) {
                    
                    $nrmarcacaoponto        = $intervalo['nrmarcacaoponto'];
                    $dataInicioFimProgOcor  = $intervalo['data'];
                    
                    $diaMarcacao = DateUtil::getDiaSemana($dataInicioFimProgOcor->format('d/m/Y'));
                    foreach($diaSemana as $key => $dia) {
                        if($key == $diaMarcacao){
                            $diaMarcacao = $dia;
                        }
                    }
                    
                    $ocorrencia = new $this->ptoProgOcorrencia;
                    $ocorrencia->setNrorg($nrorg);
                    $ocorrencia->setIdprogocorconfirmada('S');
                    $ocorrencia->setNrvinculom($vinculo);
                    $ocorrencia->setDtinicioprogocor($dataInicioFimProgOcor);
                    $ocorrencia->setDtfimprogocor($dataInicioFimProgOcor);
                    $ocorrencia->setIddiassemana($diaMarcacao);
                    $ocorrencia->setNrhordiam($alteEscala['NRHORDIAM']);
                    $ocorrencia->setNrintervalohorario($nrintervalo);
                    $ocorrencia->setIdtipohorario('N');
                    $ocorrencia->setNrtipoprogocorrencia(20);
                    $ocorrencia->setNrprogocorrencia(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_PROGOCORRENCIA', 1, 12, true));
                    $ocorrencia->setNrmarcacaoponto($nrmarcacaoponto);
                    $ocorrencia->setDtinclusao(DateUtil::getDataAtual());
                    $ocorrencia->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $ocorrencia->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                    $ocorrencia->setDtultatu(DateUtil::getDataAtual());
                    $ocorrencia->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $ocorrencia->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                    $this->entityManager->persist($ocorrencia);
                    $this->entityManager->flush($ocorrencia);
                }
                unset($horarioMarcacoes);
            }
            
            $contVinc++;                                       
        }
        
        $mensagem['sucesso'] = 'Importação de '.$contMarc.' marcações concluída para '.$contVinc.' vínculos.';
        
        return $mensagem;
    }
    
    /**
     * De acordo com os intervalos cadastrados para o horario previsto para o dia da batida, decide a sinalizacao da marcacao
     *
     * @return string (IMPORTACAO_DIA_CORRENTE, IMPORTACAO_DIA_SEGUINTE, IMPORTACAO_DIA_ANTERIOR)
     */
    private function getIdDiaMarcImportada(string $nrVinculo, \DateTime $dtHrMarc): string {
        /* Deteccao de virada para o dia anterior */
        
        $nrHordiamVinculoDiaAnterior = $this->getNrHordiamVinculoDiaAnterior($nrVinculo, $dtHrMarc);
        
        if ($this->isMarcImportadaDiaAnterior($dtHrMarc, $nrHordiamVinculoDiaAnterior)) {
            return self::IMPORTACAO_DIA_ANTERIOR;
        }
        
        /* Deteccao de virada para o dia seguinte */
        
        $nrHordiamVinculoDiaSeguinte = $this->getNrHordiamVinculoDiaSeguinte($nrVinculo, $dtHrMarc);
        
        if ($this->isMarcImportadaDiaSeguinte($dtHrMarc, $nrHordiamVinculoDiaSeguinte)) {
            return self::IMPORTACAO_DIA_SEGUINTE;
        }
        
        /* Dia convencional */
        
        return self::IMPORTACAO_DEFAULT_DIA;
    }
    
    /**
     * Busca o horario do dia imediatamente anterior ao da batida
     * 
     * @param string    $nrVinculo
     * @param \DateTime $dtHrMarc
     * 
     * @return ?string
     */
    private function getNrHordiamVinculoDiaAnterior(string $nrVinculo, \DateTime $dtHrMarc): ?string {
        $dadosHorarioVinculo = $this->getDadosHorarioVinculo(
            $this->hcmParameters->getNrorgTrabalho(),
            $nrVinculo,
            DateUtil::subtraiIntervalo($dtHrMarc, 1)->format(DateUtil::FORMATO_DATA),
            $this->hcmParameters->getDtCompetencia()
        );
        
        return $dadosHorarioVinculo['NRHORDIAM'] ?? null;
    }
    
    /**
     * Busca o horario do dia imediatamente seguinte ao da batida
     * 
     * @param string    $nrVinculo
     * @param \DateTime $dtHrMarc
     * 
     * @return ?string
     */
    private function getNrHordiamVinculoDiaSeguinte(string $nrVinculo, \DateTime $dtHrMarc): ?string {
        $dadosHorarioVinculo = $this->getDadosHorarioVinculo(
            $this->hcmParameters->getNrorgTrabalho(),
            $nrVinculo,
            DateUtil::adicionaIntervalo($dtHrMarc, 1)->format(DateUtil::FORMATO_DATA),
            $this->hcmParameters->getDtCompetencia()
        );
        
        return $dadosHorarioVinculo['NRHORDIAM'] ?? null;
    }
    
    /**
     * Verifica se a marcacao sera sinalizada como anterior considerando o horario do dia anterior
     * 
     * @param \DateTime $dtHrMarc
     * @param ?string   $nrHordiam
     * 
     * @return ?string
     */
    private function isMarcImportadaDiaAnterior(\DateTime $dtHrMarc, ?string $nrHordiam): bool {
        $naoExisteViradaNoHorario = !$this->horarioRepository->horarioIncluiDiaAnterior($this->hcmParameters->getNrorgTrabalho(), $nrHordiam);
            
        $horarioNaoInformado = empty($nrHordiam);
        
        if ($naoExisteViradaNoHorario || $horarioNaoInformado) {
            return false;
        }
        
        return $this->isMarcImportadaDentroToleranciaDiaAnterior($dtHrMarc, $nrHordiam);
    }
    
    /**
     * Verifica se a marcacao sera sinalizada como seguinte considerando o horario do dia seguinte
     * 
     * @param \DateTime $dtHrMarc
     * @param ?string   $nrHordiam
     * 
     * @return ?string
     */
    private function isMarcImportadaDiaSeguinte(\DateTime $dtHrMarc, ?string $nrHordiam): bool {
        $naoExisteViradaNoHorario = !$this->horarioRepository->horarioIncluiDiaSeguinte($this->hcmParameters->getNrorgTrabalho(), $nrHordiam);
            
        $horarioNaoInformado = empty($nrHordiam);
        
        if ($naoExisteViradaNoHorario || $horarioNaoInformado) {
            return false;
        }
        
        return $this->isMarcImportadaDentroToleranciaDiaSeguinte($dtHrMarc, $nrHordiam);
    }
    
    /**
     * Verifica se a marcacao esta dentro do periodo previsto para ser sinalizada como anterior, considerando um tempo definido de tolerancia a partir da ultima batida do horario
     * 
     * @param \DateTime $dtHrMarc
     * @param string    $nrHordiam
     * 
     * @return bool
     */
    private function isMarcImportadaDentroToleranciaDiaAnterior(\DateTime $dtHrMarc, string $nrHordiam): bool {
        $horaUltimaMarcacao = $this->horarioRepository->getHoraUltimaMarcacaoHorario($this->hcmParameters->getNrorgTrabalho(), $nrHordiam);
        $ultimaMarcacaoDia = $this->getMarcacaoFromHrIntervalo($dtHrMarc, $horaUltimaMarcacao);
        
        $toleranciaDiaAnterior = DateUtil::adicionaIntervalo(
            $ultimaMarcacaoDia,
            DateUtil::getHoraFloatDeString(self::IMPORTACAO_TEMPO_TOLERANCIA_ANTERIOR),
            DateUtil::HORAS
        );
        
        return $dtHrMarc <= $toleranciaDiaAnterior;
    }
    
    /**
     * Verifica se a marcacao esta dentro do periodo previsto para ser sinalizada como seguinte, considerando um tempo definido de tolerancia a partir da primeira batida do horario
     * 
     * @param \DateTime $dtHrMarc
     * @param string    $nrHordiam
     * 
     * @return bool
     */
    private function isMarcImportadaDentroToleranciaDiaSeguinte(\DateTime $dtHrMarc, string $nrHordiam): bool {
        $horaPrimeiraMarcacao = $this->horarioRepository->getHoraPrimeiraMarcacaoHorario($this->hcmParameters->getNrorgTrabalho(), $nrHordiam);
        $primeiraMarcacaoDia = $this->getMarcacaoFromHrIntervalo($dtHrMarc, $horaPrimeiraMarcacao);
        
        $toleranciaDiaSeguinte = DateUtil::subtraiIntervalo(
            $primeiraMarcacaoDia,
            DateUtil::getHoraFloatDeString(self::IMPORTACAO_TEMPO_TOLERANCIA_SEGUINTE),
            DateUtil::HORAS
        );
        
        /* Caso o horario seja somente no dia seguinte ou no dia anterior */
        
        if (!$this->horarioRepository->horarioIncluiDiaCorrente($this->hcmParameters->getNrorgTrabalho(), $nrHordiam)) {
            $toleranciaDiaSeguinte = DateUtil::adicionaIntervalo(
                $toleranciaDiaSeguinte,
                1,
                DateUtil::DIA
            );
        }
        
        return $dtHrMarc >= $toleranciaDiaSeguinte;
    }
    
    /**
     * Soma a hora a data informada
     * 
     * @param \DateTime $dtHrMarc
     * @param ?float    $hora
     * 
     * @return \DateTime
     */
    private function getMarcacaoFromHrIntervalo(\DateTime $dtHrMarc, ?float $hora): \DateTime {
        $dataFinal = DateUtil::adicionaIntervalo(
            DateUtil::truncateData($dtHrMarc),
            floor($hora),
            DateUtil::HORAS
        );
        
        $dataFinal = DateUtil::adicionaIntervalo(
            $dataFinal,
            round(($hora - floor($hora)) * 60), // Decimal para minutos
            DateUtil::MINUTOS
        );
        
        return $dataFinal;
    }
    
    /**
     *  Rotina criada com a função de retornar índices de array considerados como inícios de intervalos de descanso a partir da quantidade de marcações
     * em um determinado dia passado como parâmetro
     * 
     *  @param  int     $quantidadeMarcacoes
     * 
     *  @return Array   $nums
     *  Retorna numeros pares > 0 antes do número passado -1. Os números são retornados como índices de array, então como número -1.
     *  Ex. 10: [2, 4, 6] (pares antes de 9) => [1, 3, 5]
     */
    public function getIndexesPossiveisDescanso($quantidadeMarcacoes) {
        $nums=[];
        for ($k=2; $k<$quantidadeMarcacoes-1; $k+=2) {
            $nums[]=$k-1;
        }
        return $nums;
    }
    
    /**
     *  Rotina responsável por calcular o tempo trabalhado por um determinado vínculo em um determinado dia
     * 
     *  @param  $nrvinculom
     *  @param  $data
     * 
     *  @return Float
     */
    public function calculateHorasTrabalhadas($nrVinculo, $data) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();

        $pares = $this->getParesMarcacoesDia($nrVinculo, $data);
        
        $trabalhadoTime = 0;

        foreach ($pares as $par) {
            $trab = 0;

            $consideraTempo =
                $par['IDMARCACAO'] === self::LABEL_IDTIPOINTERVALO_TRABALHO ||
                $par['NRPROGOCORRENCIA_JUSTIFICADA'] !== '';

            if ($consideraTempo) {
                $trab = DateUtil::getHoraFloatDeString($par['TEMPO']);
            }
            
            $trabalhadoTime += $trab;
        }

        return $trabalhadoTime;
    }
    
    /**
     *  Rotina responsável por retornar as marcações em pares, indicando se o intervalo é de trabalho ou não
     * 
     *  @param  $nrvinculom
     *  @param  $data
     * 
     *  @return Array
     */
    public function getParesMarcacoesDia($nrvinculom, $data) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();

        $params = [
            'P_NRORG'           => $nrorg,
            'P_NRVINCULOM'      => $nrvinculom,
            'P_DTAPURACAOPONTO' => $data,
        ];

        $sql = <<<SQL
            SELECT
              ROWNUM,
              NRMARCACAOPONTO,
              MARC.NRVINCULOM,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS DTHRAJUSTADA,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
              DTHRAJUSTADA AS DTHRORDER,
              IDMARCDESPREZADA,
              IDDIAOCORRENCIA,
              DSJUSTIFICATIVA,
              COL.NRCOLETOR,
              COL.DSCOLETOR,
              IDREFERINTERVALO
             FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
               ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND MARC.NRVINCULOM = :P_NRVINCULOM
              --AND MARC.IDMARCDESPREZADA = 'N'
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA = 'DC')
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                   OR (IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY DTHRAJUSTADA, NRMARCACAOPONTO
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute($params);

        $marcacoesPonto = $query->fetchAll();
        $intervalos = $this->factoryIntervaloMarcacoes($marcacoesPonto, $data, $nrvinculom);

        if (isset($intervalos)) {
            return $intervalos;
        } else {
            return null;
        }
    }
    
    public function factoryIntervaloMarcacoes($marcacoesPonto, $dtapuracaoponto, $nrvinculom) {

        $intervalosHorario = array();

        // gera os intervalos de entrada e calcula o tempo
        $intervalosEntrada = $this->getIntervalosHorario($marcacoesPonto, self::LABEL_IDTIPOINTERVALO_TRABALHO, $dtapuracaoponto, $nrvinculom);

        // remove as marcações de primeira entrada (chegada) e última sáida (embora)
        unset($marcacoesPonto[0]);
        array_pop($marcacoesPonto);

        // gera os intervalos de saída e calcula o tempo
        $intervalosSaida = $this->getIntervalosHorario($marcacoesPonto, self::LABEL_IDTIPOINTERVALO_AUSENCIA, $dtapuracaoponto, $nrvinculom);

        // reordena o array pelo o horário, colocando-o como chave para facilitar a ordenação.
        $pares = [];
        $qtEntradas = count($intervalosEntrada);

        for ( $i = 0; $i < $qtEntradas; $i++) {
            array_push($pares, $intervalosEntrada[$i]);

            if (isset($pares, $intervalosSaida[$i])) {
                array_push($pares, $intervalosSaida[$i]);
            }
        }

        return $pares;
    }

    private function getIntervalosHorario($marcacoesPonto, $idmarcacao, $dtapuracaoponto, $nrvinculom) {

        $paresMarcacoes = array_chunk($marcacoesPonto, 2);

        $intervalos = array();

        

        foreach ($paresMarcacoes as $key => $intervalosMarcacoes) {
            /* Posição 0 do array "$intervalosMarcacoes" são marcações de início
             * Posição 1 do array "$intervalosMarcacoes" são marcações de fim
             */

            // garante que foi retornado um intervalo com duas marcações
            if (count($intervalosMarcacoes) == 2) {
                $intervalos[$key]["NRMARCACAOPONTO"]       = $intervalosMarcacoes[0]["NRMARCACAOPONTO"];
                $intervalos[$key]["NRMARCACAOPONTO_SAIDA"] = $intervalosMarcacoes[1]["NRMARCACAOPONTO"];

                $intervalos[$key]["IDMARCACAO"]      = $idmarcacao;
                $intervalos[$key]["DTMARCACAOPONTO"] = $intervalosMarcacoes[0]["DTMARCACAOPONTO"];

                $intervalos[$key]["DTHRINICIO"] = $intervalosMarcacoes[0]["HORA_MINUTO"];
                $intervalos[$key]["DTHRFIM"]    = $intervalosMarcacoes[1]["HORA_MINUTO"];

                $intervalos[$key]["DIA_INICIO"] = $intervalosMarcacoes[0]["IDDIAOCORRENCIA"];
                $intervalos[$key]["DIA_FIM"]    = $intervalosMarcacoes[1]["IDDIAOCORRENCIA"];

                $horaEntrada    = $intervalosMarcacoes[0]["HORA_MINUTO"];
                $horaSaida      = $intervalosMarcacoes[1]["HORA_MINUTO"];
                $tempoIntervalo = DateUtil::subtrairHoras($horaSaida, $horaEntrada);
                $intervalos[$key]["TEMPO"] = $tempoIntervalo;

                $intervalos[$key]["IDREFERINTERVALO_INI"] = $intervalosMarcacoes[0]["IDREFERINTERVALO"];
                $intervalos[$key]["IDREFERINTERVALO_FIM"] = $intervalosMarcacoes[1]["IDREFERINTERVALO"];

                $identificaIntervalo = $this->verificaIdentificaIntervalo($intervalosMarcacoes[0]);
                if (isset($identificaIntervalo["NRPROGOCORRENCIA"])) {
                    $intervalos[$key]["ISALMOCO"] = "true";
                    $intervalos[$key]["NRPROGOCORRENCIA_ALMOCO"] = $identificaIntervalo["NRPROGOCORRENCIA"];
                } else {
                    $intervalos[$key]["ISALMOCO"] = "false";
                    $intervalos[$key]["NRPROGOCORRENCIA_ALMOCO"] = '';
                }

                $abonarSaida = $this->verificaAbonarSaida($intervalosMarcacoes[0]);

                if (isset($abonarSaida["NRPROGOCORRENCIA"])) {
                    $intervalos[$key]["ISJUSTIFICADA"] = "true";
                    $intervalos[$key]["NRPROGOCORRENCIA_JUSTIFICADA"]          = $abonarSaida["NRPROGOCORRENCIA"];
                    $intervalos[$key]["NRMOTIVOOCORFREQ_LABEL"]                = $abonarSaida["DSMOTIVOOCORFREQ"];
                    $intervalos[$key]["NRMOTIVOOCORFREQ"]                      = $abonarSaida["NRMOTIVOOCORFREQ"];
                    $intervalos[$key]["NRPROGOCORRENCIA_DSJUSTIFICATIVA"]      = $abonarSaida["DSOBSERVACAO"];
                    $intervalos[$key]["NRPROGOCORRENCIA_DSOBSERVACAORECUSA"]   = $abonarSaida["DSOBSERVACAORECUSA"];
                    $intervalos[$key]["NRPROGOCORRENCIA_IDPROGOCORCONFIRMADA"] = $abonarSaida["IDPROGOCORCONFIRMADA"];
                    $intervalos[$key]["NRPROGOCORRENCIA_DSPATH"]               = $abonarSaida["DSPATH"];

                    if (isset($abonarSaida["IDTIPOINTERVALO"]) && $abonarSaida["IDTIPOINTERVALO"] == self::IDTIPOINTERVALO_TRABALHO)
                        $intervalos[$key]["IDMARCACAO"] = self::LABEL_IDTIPOINTERVALO_TRABALHO;
                } else {
                    $intervalos[$key]["ISJUSTIFICADA"] = "false";
                    $intervalos[$key]["NRPROGOCORRENCIA_JUSTIFICADA"]          = '';
                    $intervalos[$key]["NRMOTIVOOCORFREQ_LABEL"]                = '';
                    $intervalos[$key]["NRMOTIVOOCORFREQ"]                      = '';
                    $intervalos[$key]["NRPROGOCORRENCIA_DSJUSTIFICATIVA"]      = '';
                    $intervalos[$key]["NRPROGOCORRENCIA_IDPROGOCORCONFIRMADA"] = '';
                    $intervalos[$key]["NRPROGOCORRENCIA_DSPATH"]               = '';
                }
            }
        }

        return $intervalos;

    }
    
    /**
     *  Rotina responsável por realizar o cadastro de várias marcações para vários vínculos e dias
     * 
     *  @param  string  $diaInicio
     *  @param  string  $diaFim
     *  @param  Array   $marcacaoList       | [['ENTRADA' => 'XX:XX', 'SAIDA' => 'XX:XX' 'NRCOLETOR' => INT, 'IDDIAOCORRENCIA_ENTRADA' => IN self::IDDIAOCORRENCIA_*, 'IDDIAOCORRENCIA_SAIDA' => IN self::IDDIAOCORRENCIA_*, 'SAIDA_ALMOCO' = 'N|S']]
     *  @param  Array   $nrestrutlegalList
     *  @param  Array   $nrestrutgerenList
     *  @param  Array   $nrvinculomList
     * 
     *  @return bool
     *  @throws \Exception
     */
    public function setMarcacoesCadastroAutomatico(
        string $diaInicio,
        string $diaFim,
        $marcacaoList,
        $nrestrutlegalList,
        $nrestrutgerenList,
        $nrvinculomList,
        $conformeHorarioBase
    ): bool {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        // Verifica se existem marcações a serem inseridas
        if (!is_array($marcacaoList) || count($marcacaoList) < 1 && !$conformeHorarioBase) {
            throw new \Exception(Messages::HCM07003_CADAUTO_UNSUFICCIENT_ITEMS, Messages::ALERT_MESSAGE_CODE);
        }
        
        // Valida se os filtros foram devidamente preenchidos
        if ((empty($nrvinculomList)     || !is_array($nrvinculomList)       || count($nrvinculomList)       < 1) &&
            (empty($nrestrutlegalList)  || !is_array($nrestrutlegalList)    || count($nrestrutlegalList)    < 1) &&
            (empty($nrestrutgerenList)  || !is_array($nrestrutgerenList)    || count($nrestrutgerenList)    < 1))
        {
            throw new \Exception(Messages::HCM07003_CADAUTO_UNSUFICCIENT_FILTERS, Messages::ALERT_MESSAGE_CODE);
        }
        
        $nrvinculomList     = empty($nrvinculomList)    ? null : $nrvinculomList;
        $nrestrutlegalList  = empty($nrestrutlegalList) ? null : $nrestrutlegalList;
        $nrestrutgerenList  = empty($nrestrutgerenList) ? null : $nrestrutgerenList;
        
        // Verifica se há vínculos nos filtros, considerando a lista de vínculos e de estruturas
        $vinculos = $this->vinculoRepository->getVinculosByEstruturas($nrorg, $dtmescompetenc, $nrestrutlegalList, $nrestrutgerenList, $nrvinculomList);
        if (empty($vinculos) || !is_array($vinculos) || count($vinculos) < 1) {
            throw new \Exception(Messages::HCM00000_GER_VINC_N_ENCONTR_FILTRO_INF_ERROR, Messages::ALERT_MESSAGE_CODE);
        }
        
        $dataInicio     = DateUtil::getDataDeString($diaInicio, DateUtil::FORMATO_BRASILEIRO, true);
        $dataFim        = DateUtil::getDataDeString($diaFim, DateUtil::FORMATO_BRASILEIRO, true);
        $dataCorrente   = $dataInicio;
        
        // Armazena as marcaçõe que serão relacionadas à ocorrências de almoço
        $nrMarcacaoPontoSaidaInseridas = [];
        $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco = [];

        $this->entityManager->beginTransaction();

        try {

            // Passa dia a dia, inserindo as marcações para os vínculos filtrados
            while ($dataCorrente <= $dataFim) {
                // $dataString = $dataCorrente->format(DateUtil::FORMATO_BRASILEIRO);
                $vinculos = $this->vinculoRepository->getVinculosByEstruturas($nrorg, $dtmescompetenc, $nrestrutlegalList, $nrestrutgerenList, $nrvinculomList);
                if (empty($vinculos) || !is_array($vinculos) || count($vinculos) < 1) {
                    throw new \Exception(Messages::HCM00000_GER_VINC_N_ENCONTR_FILTRO_INF_ERROR, Messages::ALERT_MESSAGE_CODE);
                }
    
                $nrVinculos = array_map(function ($item) { return $item['NRVINCULOM']; }, $vinculos);

                if ($conformeHorarioBase) {
                    $this->setMarcacoesCadAutoPorHorario(
                        $nrVinculos, $dataCorrente
                    );
                } else {
                    $this->setMarcacoesCadAuto(
                        $nrVinculos, $dataCorrente, $marcacaoList
                    );
                }
                
                $dataCorrente = DateUtil::adicionaIntervalo($dataCorrente);
            }

            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }

        /* Inclui as marcacoes de almoco conforme as informacoes armazenadas na criacao das marcacoes */
        $dadosMarcacaoesSaida = array_map(
            function ($item) {
                return [
                    'NRVINCULOM' => $item->getNrvinculom(),
                    'NRMARCACAOPONTO' => $item->getNrMarcacaoponto(),
                ];
            },
            $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco
        );

        \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance()->getService('\HCM\PTO\Services\OcorrenciaFrequenciaService')->geraOcorrenciasAlmoco($dadosMarcacaoesSaida);
        
        return true;
    }

    /**
     *  Cria uma lista de marcacoes para um conjunto de vinculos
     */
    public function setMarcacoesCadAuto(
        array $nrVinculos,
        \DateTime $data,
        array $marcacaoList
    ): void {
        $dataString = $data->format(DateUtil::FORMATO_BRASILEIRO);
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        /* Percorre os intervalos informados */
        foreach ($marcacaoList as $intervalo) {
            $entrada    = $intervalo['ENTRADA'] ?? null;
            $saida      = $intervalo['SAIDA']   ?? null;
            $nrcoletor  = $intervalo['NRCOLETOR'] ?? null;
            $iddiaocorrenciaEntrada = $intervalo['IDDIAOCORRENCIA_ENTRADA'] ?? null;
            $iddiaocorrenciaSaida   = $intervalo['IDDIAOCORRENCIA_SAIDA'] ?? null;
            $adicionaOcorrenciaAlmoco = !empty($intervalo['SAIDA_ALMOCO']) && $intervalo['SAIDA_ALMOCO'] == 'S';
            
            $dataHoraEntrada = DateUtil::getDateTimeFromDataHoraString($dataString, $entrada);
            $dataHoraSaida = !empty($saida) ? DateUtil::getDateTimeFromDataHoraString($dataString, $saida) : null;

            /* Para cada intervalo cria as marcacoes para cada vinculo */
            foreach ($nrVinculos as $nrVinculo) {
                if (!empty($dataHoraEntrada)) {
                    $this->criarMarcacao(
                        $nrVinculo, $dataHoraEntrada, $nrcoletor, $iddiaocorrenciaEntrada
                    );
                }

                if (!empty($dataHoraSaida)) {
                    $marcacaoSaida = $this->criarMarcacao(
                        $nrVinculo, $dataHoraSaida, $nrcoletor, $iddiaocorrenciaSaida
                    );
                    
                    if (!empty($marcacaoSaida) && $adicionaOcorrenciaAlmoco) {
                        $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco[] = $marcacaoSaida;
                    }
                }
            }
        }
    }

    /**
     * Replica as marcacoes previstas no horario para um conjunto de vinculos
     */
    public function setMarcacoesCadAutoPorHorario(
        array $nrVinculos,
        \DateTime $data
    ): void {

        $dataString = $data->format(DateUtil::FORMATO_BRASILEIRO);
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        foreach($nrVinculos as $nrVinculo){
            $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVincMestreHistorico($nrorg, $nrVinculo, DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y'));

            $situacao = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoPorDatainicio($nrorg, $nrVinculo, $data->format('d/m/Y'));            
            $situacaoh = null;
            /* Verifica se o vínculo está afastado ou de atestado*/
            if(!empty($situacao)) {
                $situacaoh = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->getAlteSitufunc(DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y'), $nrorg, $situacao->getNrsitufuncm());
                $idafastamento = $situacaoh[0]['IDAFASTSITUFUNC'];
                $nrsitufunc = $situacaoh[0]['NRSITUFUNCM'];
            }
            if(!empty($situacaoh) && ($idafastamento == 'S' || $nrsitufunc == '12' || $nrsitufunc == '15' ||$nrsitufunc == '16' ||$nrsitufunc == '17')) {
                continue;
            }

            /* Verifica se o vínculo se enquadra no artigo 62 da CLT */
            $escala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->getAlteEscalaByCompetencia($nrVinculo, $nrorg, DateUtil::getPrimeiroDiaDoMes($data)->format('d/m/Y')); 
            if($escala['IDCONTJORNADA'] == 'S') {
                continue;
            }

            /* Verifica se o vínculo está de Férias */
            $ferias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaItem($nrorg, $nrVinculo);
            $emFerias = false;
            foreach($ferias as $gozo) {
                // Verifica se a data da marcação está entre um período de gozo ou é igual ao começo ou final de gozo
                if((DateUtil::comparaDatas($data, $gozo->getDtinigozoferias(), 2) && DateUtil::comparaDatas($data, $gozo->getDtfimgozoferias(), 1)) 
                || (DateUtil::comparaDatas($data, $gozo->getDtinigozoferias(), 3) || DateUtil::comparaDatas($data, $gozo->getDtfimgozoferias(), 3))) {
                    $emFerias = true;
                }
            }
            if($emFerias) {
                continue;
            }
            
            /* Horario do dia em questao */
            $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrorg, $nrVinculo, $dataString, $this->hcmParameters->getDtCompetencia());
            if(empty($dadosHorarioVinculo)){
                continue;
            }
            
            /* Verifica se o dia eh feriado 
               Caso o tipo do horário seja turno, o processo deve continuar */
            if ($this->calendarioService->isFeriadoVinculo($nrVinculo, $dataString) && $dadosHorarioVinculo['IDTIPOHORARIO'] != "T") {
                continue;
            }

            /* Busca os intervalos do horario */
            $intervalos = $this->horarioRepository->getIntervalosHorario($nrorg, $dadosHorarioVinculo['NRHORDIAM'] ?? null);
            
            /* Busca pela ocorrência de abono de falta no dia em questão e o exclui */
            if($dadosHorarioVinculo['IDTIPOHORARIO'] == "T") {
                $ocorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array("nrorg" => $nrorg, "nrvinculom" => $nrVinculo, "dtinicioprogocor" => $data, "dtfimprogocor" => $data));
                if(!empty($ocorrencia)) {
                    $this->entityManager->remove($ocorrencia);
                }
            }
            
            
            /* Cria as marcacoes de acordo com os intervalos */
            foreach($intervalos as $intervalo){
                $dataHoraEntrada = $this->getMarcacaoFromHrIntervalo($data, DateUtil::getHoraFloatDeString($intervalo->getHrentrada()));
                $iddiaocorrenciaEntrada = $this->translateIdDiaOcorrenciaFromHorario($intervalo->getIddiaocorreentrada());

                /* Somente intervalos com controle 'Normal' sao considerados */
                if ($intervalo->getIdctrlmarcentrada() == self::IDCTRLMARC_NORMAL) {
                    $this->criarMarcacao(
                        $nrVinculo, $dataHoraEntrada, self::CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT, $iddiaocorrenciaEntrada
                    );
                }

                $dataHoraSaida = $this->getMarcacaoFromHrIntervalo($data, DateUtil::getHoraFloatDeString($intervalo->getHrsaida()));
                $iddiaocorrenciaSaida = $this->translateIdDiaOcorrenciaFromHorario($intervalo->getIddiaocorresaida());

                /* Somente intervalos com controle 'Normal' sao considerados */
                if ($intervalo->getIdctrlmarcsaida() == self::IDCTRLMARC_NORMAL) {
                    $marcacaoSaida = $this->criarMarcacao(
                        $nrVinculo, $dataHoraSaida, self::CADAUTOMARC_DEFAULT_NRCOLETOR_DEFAULT, $iddiaocorrenciaSaida
                    );
    
                    /* Armazena a primeira marcacao de saida para manipulacoes posteriores */
                    if (!empty($marcacaoSaida) && $intervalo->getNrseqintervhorario() == 1 && count($intervalos) > 1) {
                        $this->marcacaoesSaidasCriadasCadAutomSinalizacaoAlmoco[] = $marcacaoSaida;
                    }
                }
            }

        }
    }

    /**
     * Traduz o iddiaocorrencia vindo do intervalo para o utilizado nas marcacoes
     */
    public function translateIdDiaOcorrenciaFromHorario(
        string $idDiaOcorrencia
    ): string {

        switch ($idDiaOcorrencia){
            case self::IDDIAOCORRENCIA_DIA_CORRENTE:
                return self::IDDIAOCORRENCIA_FORCA_DIA_CORRENTE;
            case self::IDDIAOCORRENCIA_DIA_ANTERIOR:
                return self::IDDIAOCORRENCIA_FORCA_DIA_ANTERIOR;
            case self::IDDIAOCORRENCIA_DIA_SEGUINTE:
                return self::IDDIAOCORRENCIA_FORCA_DIA_SEGUINTE;
        }
    }

    /**
     * Retorna o dia no qual uma marcacao faz referencia, o dia no qual os calculos de frequencia serao
     * gerados
     */
    public function getDataApuracaoMarcacao(
        \DateTime $dataMarcacao,
        string $iddiaocorrencia
    ): \DateTime {

        if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_ANTERIORES)) {
            return DateUtil::subtraiIntervalo($dataMarcacao, 1);
        } else if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_SEGUINTES)) {
            return DateUtil::adicionaIntervalo($dataMarcacao, 1);
        }
        return $dataMarcacao;
    }

    /**
     * De acordo com a ocorrencia retorna o dia ajustado correspondente
     */
    public function alteraDataDeAcordoComDiaOcorrencia(
        \DateTime $dataMarcacao,
        string $iddiaocorrencia
    ): \DateTime {

        if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_ANTERIORES)) {
            return DateUtil::adicionaIntervalo($dataMarcacao, 1);
        } else if (in_array($iddiaocorrencia, self::IDDIAOCORRENCIA_SEGUINTES)) {
            return DateUtil::subtraiIntervalo($dataMarcacao, 1);
        }
        return $dataMarcacao;
    }

    /**
     * Cria uma entidade GpeMarcacaoponto
     */
    public function criarMarcacao(
        string $nrvinculom,
        \DateTime $dataHora,
        string $nrcoletor,
        string $iddiaocorrencia
    ): ?GpeMarcacaoponto {

        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $cdoperador = $this->hcmParameters->getCdoperador();
        
        /* Altera a data da marcacao de acordo com a indicacao do dia da ocorrencia, para que a marcacao faca referencia a data original */
        $dataHora = $this->alteraDataDeAcordoComDiaOcorrencia($dataHora, $iddiaocorrencia);

        /* Validacao para impedir a criacao de marcacoes duplicadas */
        $marcacaoExistente = $this->entityManager->getRepository(Repositories::GPE_MARCACAOPONTO)->findOneBy(array(
            "nrorg" => $nrorg, "nrvinculom" => $nrvinculom, "dthrajustada" => $dataHora
        ));
        if(!empty($marcacaoExistente)) {
            $this->msgProcesso->addMessage(
                "Marcação: '{$dataHora->format(DateUtil::FORMATO_DATAHORA)}' não criada pois já existe uma marcação com mesma data e horário. - Vínculo: $nrvinculom"
            );
            return null;
        }
            
        $nrmarcacaoponto = NovoCodigo::geraCodigo($nrorg, 'GPE_MARCACAOPONTO', 1, 12, true);
        
        $novaMarcacaoPonto = new $this->gpeMarcacaoponto;
        $novaMarcacaoPonto->setNrvinculom($nrvinculom);
        $novaMarcacaoPonto->setNrmarcacaoponto($nrmarcacaoponto);
        $novaMarcacaoPonto->setNrorg($nrorg);
        $novaMarcacaoPonto->setDthrmarcacaoponto($dataHora);
        $novaMarcacaoPonto->setDthrajustada($dataHora);
        $novaMarcacaoPonto->setNrcoletor($nrcoletor);
        $novaMarcacaoPonto->setIddiaocorrencia($iddiaocorrencia);
        $novaMarcacaoPonto->setIdenquadrahorario(self::IDENQUADRAHORARIO_DEFAULT_ONCREATION);
        $novaMarcacaoPonto->setIdmarcdesprezada(self::MARCDESPREZADA_DEFAULT_ONCREATION);
        $novaMarcacaoPonto->setIdativo('S');
        $novaMarcacaoPonto->setDtinclusao(DateUtil::getDataAtual());
        $novaMarcacaoPonto->setCdoperinclusao($cdoperador);
        $this->entityManager->persist($novaMarcacaoPonto);

        return $novaMarcacaoPonto;
    }
    
    public function getMarcacaoPontoEspelho($filter) {
        $params = [
            "P_NRORG" => isset($filter['P_NRORG']) ? $filter['P_NRORG'] : null,
            "P_NRPERIODOAPURACAO" => isset($filter['NRPERIODOAPURACAO']) ? $filter['NRPERIODOAPURACAO'] : null,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia()
        ];

        $cond = "";
        
        if(!empty($filter['NRVINCULOM'])) {
            $vinculo = $filter['NRVINCULOM'];
            $cond .= "AND RES.NRVINCULOM IN ( $vinculo )";
        }
        if(!empty($filter['NRESTRUTLEGAL'])) {
            $estrutLegal = $filter['NRESTRUTLEGAL'];
            $cond .= "AND VH.NRESTRUTLEGAL IN ( $estrutLegal )";
        }
        if(!empty($filter['NRESTRUTGEREN'])) {
            $estrutGeren = $filter['NRESTRUTGEREN'];
            $cond .= "AND VH.NRESTRUTGEREN IN ( $estrutGeren )";
        }
        if(!empty($filter['NRCOLETOR_LIST'])) {
            $coletor = $filter['NRCOLETOR_LIST'];
            $cond .= "AND RES.NRCOLETOR IN ( $coletor )";
        } else {
            $cond .= "AND RES.NRCOLETOR NOT IN (21,22,23,25)";
        }
        
        $cond .= "ORDER BY RES.NRVINCULOM, RES.DIA_REFERENCIA";
        
        $sql = "
            WITH
            MARCACOES AS (
            SELECT
                MARC.NRVINCULOM,
                MARC.DTHRMARCACAOPONTO,
                MARC.DTHRAJUSTADA,
                MARC.IDDIAOCORRENCIA,
                MARC.NRCOLETOR,
                PA.NRPERIODOAPURACAO,
                MARC.NRORG,
                CASE
                    WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                        TRUNC(MARC.DTHRAJUSTADA, 'DD') - 1
                    WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                        TRUNC(MARC.DTHRAJUSTADA, 'DD') + 1
                    ELSE
                        TRUNC(MARC.DTHRAJUSTADA, 'DD')
                END AS DIA_REFERENCIA
            FROM GPE_MARCACAOPONTO MARC
            JOIN PTO_PERIODOAPURACAO PA ON PA.NRORG             = MARC.NRORG
                                        AND PA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
            WHERE MARC.NRORG       = :P_NRORG
                AND MARC.NRCOLETOR NOT IN (21,22,23,25)
                AND (CASE
                        WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                            TRUNC(MARC.DTHRAJUSTADA) - 1
                        WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                            TRUNC(MARC.DTHRAJUSTADA) + 1
                        ELSE
                            TRUNC(MARC.DTHRAJUSTADA)
                    END
                ) BETWEEN PA.DTINICIALAPURACAO AND PA.DTFINALAPURACAO
            ),

            IDS AS (
            SELECT
                MARCACOES.NRVINCULOM,
                MARCACOES.DTHRAJUSTADA,
                MARCACOES.DIA_REFERENCIA,
                MARCACOES.NRPERIODOAPURACAO,
                MARCACOES.NRORG,
                MARCACOES.NRCOLETOR,
                ROW_NUMBER() OVER(
                    PARTITION BY NRVINCULOM, DIA_REFERENCIA
                    ORDER BY DTHRAJUSTADA
                ) NRO_MARCACAO
            FROM MARCACOES
            ORDER BY DTHRAJUSTADA
            ),

            RES AS(
                SELECT
                    *
                FROM IDS
                PIVOT (
                    MIN( DTHRAJUSTADA )
                    FOR NRO_MARCACAO
                    IN (
                        '1' DATA_INICIO,
                        '2' DATA_FIM,
                        '3' DATA_INICIO2,
                        '4' DATA_FIM2,
                        '5' DATA_INICIO3,
                        '6' DATA_FIM3,
                        '7' DATA_INICIO4,
                        '8' DATA_FIM4
                    )
                )
            )

            SELECT 
                RES.NRVINCULOM, RES.DIA_REFERENCIA, 
                RES.NRPERIODOAPURACAO, 
                RES.NRORG,
                RES.DIA_REFERENCIA,
                RES.NRCOLETOR,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                TO_CHAR(RES.DATA_INICIO, 'HH24:MI') AS HORA_INICIO,
                TO_CHAR(RES.DATA_INICIO, 'DD/MM/YYYY') AS DATA_INICIO,
                TO_CHAR(RES.DATA_FIM, 'HH24:MI') AS HORA_FIM,
                TO_CHAR(RES.DATA_FIM, 'DD/MM/YYYY') AS DATA_FIM,
                TO_CHAR(RES.DATA_INICIO2, 'HH24:MI') AS HORA_INICIO2,
                TO_CHAR(RES.DATA_INICIO2, 'DD/MM/YYYY') AS DATA_INICIO2,
                TO_CHAR(RES.DATA_FIM2, 'HH24:MI') AS HORA_FIM2,
                TO_CHAR(RES.DATA_FIM2, 'DD/MM/YYYY') AS DATA_FIM2,
                TO_CHAR(RES.DATA_INICIO3, 'HH24:MI') AS HORA_INICIO3,
                TO_CHAR(RES.DATA_INICIO3, 'DD/MM/YYYY') AS DATA_INICIO3,
                TO_CHAR(RES.DATA_FIM3, 'HH24:MI') AS HORA_FIM3,
                TO_CHAR(RES.DATA_FIM3, 'DD/MM/YYYY') AS DATA_FIM3,
                TO_CHAR(RES.DATA_INICIO4, 'HH24:MI') AS HORA_INICIO4,
                TO_CHAR(RES.DATA_INICIO4, 'DD/MM/YYYY') AS DATA_INICIO4,
                TO_CHAR(RES.DATA_FIM4, 'HH24:MI') AS HORA_FIM4,
                TO_CHAR(RES.DATA_FIM4, 'DD/MM/YYYY') AS DATA_FIM4,
                VM.NMVINCULOM,
                VM.NMVINCULOM AS NMVINCULOM_NOTGROUPED
            FROM 
                RES
                JOIN GPE_VINCULOM VM
                    ON VM.NRVINCULOM = RES.NRVINCULOM
                    AND VM.NRORG = :P_NRORG
                JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                    AND VH.NRORG = VM.NRORG
                    AND VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC)
                                               FROM GPE_VINCULOH VHH
                                              WHERE VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND VHH.NRORG = VH.NRORG
                                                AND VHH.NRVINCULOM = VH.NRVINCULOM)
            WHERE
                RES.NRORG = :P_NRORG
                AND RES.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
".$cond;

        $query = $this->entityManager->getConnection()->prepare($sql);

        $query->execute($params);

        $marcacoesPonto = $query->fetchAll();
        
        return $marcacoesPonto;
    }
}
