<?php
/**
 * Created by PhpStorm.
 * User: felipesanos
 * Date: 22/07/2016
 * Time: 11:14
 */

namespace HCM\VAB\Services;

use Folha\Logic\ValesBeneficios\LancamentoValeTransporte\Lancamento;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use Folha\Entities\FpaTipomoviment;
use HCM\FPA\Services\LoteLancEventoService;

class LancamentoValeTransporteService {

    protected $hcmParameters;

    public function __construct(HCMParameters $hcmParameters, FPA $fpa, LoteLancEventoService $loteLancEventoService) {
        $this->hcmParameters = $hcmParameters;
        $this->loteLancEventoService = $loteLancEventoService;
    }

    public function verificaCalculo($nrvinculom, $dtmescompetencia, $nrorg) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_MENSAL);
            Lancamento::verificaCalculo($nrcalculofolha, $nrvinculom, $dtmescompetencia, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravarTotal($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Lancamento::grava($nrcalculofolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gravarParcial($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Lancamento::gravaParcial($nrcalculofolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $dtpgtovale, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retrocedeParcial($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Lancamento::retrocedeParcial($nrcalculofolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retrocedeTotal($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Lancamento::retrocede($nrcalculofolha, $dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $tipodelinha, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportarFolha($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom) {
        try {
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            Lancamento::exportarFolha($nrcalculofolha, $dtmescompetencia, $this->hcmParameters->getCdoperador(), $nrorg, FpaTipomoviment::TP_MOV_MENSAL, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function exportarLote($dtmescompetencia, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao, $row = null) {
        try {

            $condEstrutura = "";
            $condEstrutura = Lancamento::buildWhereEstrutura($nrestrutsind, $nrestrutlegal, $nrestrutgeren);
            
            $condAdmissao = "";
            $condAdmissao = Lancamento::buildWhereDtAdmissao($dtiniadmissao, $dtfimadmissao);
            
            $nrvinculoms = array();
            // $nrcalculofolha = FPA::determinaNrCalculoFolha($nrtpmovimento);
            $nrcalculofolhavale = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            
            // $itemmovcalcs = \Folha\Repositories\FpaItecalcfolha::getVinculosFolha($nrcalculofolhavale, $dtmescompetencia, $nrorg, $lista_nrvinculom);
            $itemmovcalcs = \Folha\Repositories\FpaItecalcfolha::getItensCalculados($nrcalculofolhavale, $nrorg, $dtmescompetencia, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $lista_lotevinculos, $dtiniadmissao, $dtfimadmissao);
            if (!$itemmovcalcs) {
                throw new \Exception("Não foi encontrado movimento de vale.",30);
            }

            $params = array(
                'P_NRORG'           => $nrorg,
                'P_NRCALCULOFOLHA'  => $nrcalculofolhavale,
                'P_DTOCORRENCIA'    => $dtmescompetencia,
                'P_DTMESCOMPETENC'  => $dtmescompetencia,
                'P_NRESTRUTSIND'     => $nrestrutsind,
                'P_NRESTRUTLEGAL'    => $nrestrutlegal,
                'P_NRESTRUTGEREN'    => $nrestrutgeren,
                'P_LISTA_NRVINCULOM'=> $lista_nrvinculom,
                'P_NRTIPOMOVIMENTVALE' => Fpatipomoviment::TP_MOV_VALE_TRANSPORTE,
                'P_LISTA_LOTE_VINCULOS' => $lista_lotevinculos,
                'P_DTINIADMISSAO' => $dtiniadmissao,
                'P_DTFIMADMISSAO' => $dtfimadmissao,
                'P_NRESTRUTLEGALOPER'    => FPA::getNrestrutlegalOper(),
                'P_NRESTRUTGERENOPER'    => FPA::getNrestrutgerenOper()
            );

            $where = "e.NRORG = :P_NRORG 
                      AND e.NRCALCULOFOLHA = (SELECT NRCALCULOFOLHA FROM FPA_CALCULOFOLHA WHERE NRCALCULOFOLHA = :P_NRCALCULOFOLHA AND NRORG = :P_NRORG AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENTVALE) 
                      AND e.NREVENTOM = 3040 
                      AND (INSTR(:P_LISTA_NRVINCULOM, '_'||e.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A') "
                      .$condEstrutura
                      .$condAdmissao;
            $itemcalcs = \Folha\Entities\FpaItecalcfolha::getObjectsByWhere($where, $params);
            if ($itemcalcs) {
                throw new \Exception("Vínculo(s) com parcial calculado. Favor calcular total para exportar para a folha.",20);
            }

            //Gera lote de eventos
            $rowObsNovoLote = !empty($row['OBSNOVOLOTE']) ? $row['OBSNOVOLOTE'] : null;
            $modificaObsLote = "$rowObsNovoLote";
            $rowDsNovoLote = !empty($row['DSNOVOLOTE']) ? $row['DSNOVOLOTE'] : null;
            $modificaDsLote = "$rowDsNovoLote";
            $dsLoteLanceventoMensal = 'EXPORTAÇÃO DE ITENS DO MOVIMENTO DE VALE TRANSPORTE';
            $idstatuslotelanc       = 'PENDENTE';
            $idoriglotelanc         = 'VAB';
            
            $this->loteLancEventoService->geraLoteLancEvento(
                $nrorg, $dtmescompetencia, $idoriglotelanc, !empty($modificaDsLote) ? $modificaDsLote : $dsLoteLanceventoMensal, $idstatuslotelanc, !empty($modificaObsLote) ? $modificaObsLote :'', FpaTipomoviment::TP_MOV_MENSAL, $itemmovcalcs 
            );
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cadastroDias($lista_nrvinculom, $nrtipolinhavinc, $qtdiasacre, $qtdiasdesc) {
        try {
            Lancamento::alteraQtdeDiasVale($lista_nrvinculom, $nrtipolinhavinc, $qtdiasacre, $qtdiasdesc);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cadastroAutomatico($nrvinculom_list, $nrlinhatransp, $nrtarifa, $nrtipolinhavinc, $nrsentidolinha, $qtdiasvale, $dtpgtovale, $dtiniapurvale, $dtfinapurvale) {
        try {
            Lancamento::cadastroAutomatico($nrvinculom_list, $nrlinhatransp, $nrtarifa, $nrtipolinhavinc, $nrsentidolinha, $qtdiasvale, $dtpgtovale, $dtiniapurvale, $dtfinapurvale);
        } catch (\Exception $e) {
            throw $e;
        }
    }

}